{******************************************************************************}
{                                                                              }
{ Excel Interface Component                                                    }
{ Component ExcelUtilities.pas                                                 }
{ This interface component can be downloaded at www.tcoq.org, component page.  }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License Version  }
{ 1.1 (the "License"); you may not use this file except in compliance with the }
{ License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ The Original Code is ExcelUtilities.pas                                      }
{                                                                              }
{ The Initial Developer of the Original Code is Thierry.Coq                    }
{ thierry.coq(at)centraliens.net                                               }
{ Portions created by this individual are Copyright (C)                        }
{ 2000-2008 of this individual.                                                }
{                                                                              }
{ Last modified: 2009/07/25                                                    }
{ Version 0.4                                                                  }
{                                                                              }
{******************************************************************************}
unit ComUtilities;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Windows, Variants, ComObj, ActiveX,
  Diagnostics;

Const
  DISPID_COUNT               =  118;
  DISPID_ITEM                =  170;
  DISPID_VISIBLE             =  558;
  DISPID_QUIT                =  302;


Type
  {TDispComponent is the root class to manage Dispinterfaces, in a manual manner
   waiting for the FPC compiler to catch up.}
  TDispComponent = class(TComponent)
  protected
    FDispatch: IDispatch;
    FHResult : HResult;
    FExceptInfo : TExcepInfo;
    // Error-handling routines
    Procedure CheckHResult;
    Procedure RaiseExternalException;

    // Low level routines
    Function BoolToVar( aValue: Boolean): TPropVariant;

    // helper procedure for calling IDispatch.Invoke.
    Procedure InvokeHelper( aDispID       : Integer;
                            aDispIDNamed  : DISPID;
                            Flags         : Word;
                            indices       : array of OleVariant;
                            aValue        : OleVariant;
                            out ArgErr    : DWord;
                            out varResult : OleVariant
                            );

    procedure PutProperty( aDispID : Integer;  anArg : TPROPVARIANT); overload;
    procedure PutProperty( aDispID : Integer;  anArg : OleVariant); overload;
    procedure PutProperty( aDispID : Integer;  indices: array of OleVariant; aValue : OleVariant); overload;
    procedure SetProperty( aName:String; aValue:Boolean); overload;
    procedure SetProperty( aPropName:String; aValue:OleVariant); overload;
    procedure SetProperty( aPropName:String; indices: array of OleVariant; aValue:OleVariant); overload;

    // GetProperty
    //=============
    //  - as a function
    function GetProperty( aName:String):OleVariant; overload;
    function GetProperty( aDispID : Integer):OleVariant; overload;

    //  - with no parameters:
    procedure GetProperty( aName:String; out aValue:IDispatch); overload;
    procedure GetProperty( aDispID : Integer; out aValue: IDispatch); overload;
    //  - with 1 parameter:
    procedure GetProperty( aName:String; index:integer; out aValue:IDispatch); overload;
    procedure GetProperty( aDispID : Integer; index:integer;out aValue: IDispatch); overload;
    //  - with 1 variant parameter:
    procedure GetProperty( aName:String; anIndex:Variant; out aValue:IDispatch); overload;
    procedure GetProperty( aDispID : Integer; anIndex:Variant; out aValue: IDispatch); overload;
    //  - with 2 parameters:
    procedure GetProperty( aName:String; i1, i2:integer; out aValue:IDispatch); overload;
    procedure GetProperty( aDispID : Integer; i1, i2:integer;out aValue: IDispatch); overload;

    // Call Procedures
    procedure CallProcedure( aName : String); overload;
    procedure CallProcedure( aDispid: Integer); overload;

    // Call procedure with a variable amount of parameters
    procedure CallProcedure( aName : String; args: array of OleVariant); overload;
    procedure CallProcedure( aDispid: Integer; args: array of OleVariant); overload;

    function CallFunctionDisp( aName : String): IDispatch; overload;
    function CallFunctionDisp( aDispid: Integer): IDispatch; overload;
    function CallFunctionDisp( aDispid: Integer; args: array of OleVariant): IDispatch; overload;

  public
    Constructor Create( anOwner : TComponent); override; overload;
    Constructor Create( anOwner : TComponent; aDispatch : IDispatch); overload;
    //Low level routines, available for object investigation, to discover
    //its interface.
    Function GetIdsOfNames( aName:String): integer;
    Procedure ShowDispid( aName:String);
    Procedure ShowMessage( aMsg: String);
    Function ExcelObject: OleVariant;
  end;

  TDispComponentClass = Class of TDispComponent;

  { TDispCollection}
  { TDispCollection is used to implement collections}
  { TODO 3 -oTC -cEvolutivity : In order to be easily used, this class should be a template class.
  We have to wait the required version of FPC.
 }
  TDispCollection=class(TDispComponent)
  protected
    ItemClassType : TDispComponentClass;
    function GetItem( index:OleVariant): TDispComponent; virtual; overload;
  { TODO 2 -oTC -cCompletion : Need to add a setter too, if possible, and the add function
    where it makes sense}
  public
    function Count: Integer;
    property Item[index:OleVariant]:TDispComponent read GetItem; default;
  end;

  { TDispServer }
  { TDispServer is used to encapsulate non-Excel specific code, for the server}
  TDispServer = class( TDispComponent)
  private
    FActive: Boolean;
    FServerName: String;
    FVisible: Boolean;
    procedure SetActive(const AValue: Boolean);
    procedure SetServerName(const AValue: String);
    procedure SetVisible(const AValue: Boolean);
  protected
    procedure CreateComServer;
  public
    Constructor Create( anOwner : TComponent); override;
    Destructor Destroy; override;
    Procedure Quit;
  published
    Property Active:Boolean read FActive write SetActive;
    Property ServerName: String read FServerName write SetServerName;
    Property Visible:Boolean read FVisible write SetVisible;
  end;



implementation

{ TDispComponent }

procedure TDispComponent.CheckHResult;
begin
  If FHResult <> 0 then
  begin
    Case FHResult of
      DISP_E_EXCEPTION : RaiseExternalException;
    else
      Raise Exception.Create( 'HResult = '+ IntToHex(FHResult,8));
    end;
  end;
end;

procedure TDispComponent.RaiseExternalException;
begin
  Raise Exception.Create( 'External software raised an exception, '
    + 'HResult = "'+ IntToHex(FHResult,8)+'", '
    + 'Exception SCode = "'+IntToHex(FExceptInfo.Scode,8)+'"'
    );
end;

function TDispComponent.GetIdsOfNames(aName: String): integer;
var
  lDispId: Integer;
  member : WideString;
begin
  member := aName;
  FHResult := FDispatch.GetIDsOfNames(GUID_NULL, @member, 1, LOCALE_USER_DEFAULT, @lDispID);
  Result := lDispId;
  logMessage('GetIDsOfNames on "'+aName+'", DispID="'+IntToStr(Result)+'", HResult =' +IntToStr(FHResult));
  CheckHResult;
end;

procedure TDispComponent.ShowDispid(aName: String);
var
  aDispid : integer;
  aMsg : String;
begin
  aDispid := GetIdsOfNames( aName);
  aMsg := 'DISPID for procedure "'+aName+'" ='+ IntToStr(aDispid);
  ShowMessage( aMsg);
end;

procedure TDispComponent.ShowMessage(aMsg: String);
begin
  MessageBox(0, PChar(aMsg), 'Options', mb_YesNo +mb_ICONWARNING);
end;

function TDispComponent.ExcelObject: OleVariant;
begin
  result := Self.FDispatch;
end;

function TDispComponent.BoolToVar(aValue: Boolean): TPropVariant;
begin
  Result.VT := VT_BOOL;
  Result.boolVal:= aValue;
end;

procedure TDispComponent.InvokeHelper(aDispID: Integer; aDispIDNamed: DISPID;
  Flags: Word; indices: array of OleVariant; aValue: OleVariant;
  out ArgErr: DWord; out varResult: OleVariant);
var
  Params: TDispParams;
  LocalVariantArg : array of OleVariant;
  dispidNamed :  DISPID;
  iParam : integer;
  ParamNumber : Integer;
  IndicesNumber: Integer;
  ParamIncrement: Integer;
begin
  // Initialization
  dispidNamed := aDispIDNamed;
  {$HINTS OFF}
  FillChar(FExceptInfo, SizeOf(ExcepInfo),0);
  FillChar(Params, SizeOf(DispParams),0);
  {$HINTS ON}

  // Setting up parameter list
  ParamIncrement:=0;
  IndicesNumber := Length(indices);
  ParamNumber := IndicesNumber;
  if aDispIDNamed=DISPID_PROPERTYPUT then       // add one for the property_put
    inc(ParamIncrement);                        // value.
  ParamNumber := IndicesNumber + ParamIncrement;

  if ParamNumber>0 then
  begin
    SetLength( LocalVariantArg, ParamNumber);
    for iParam := 0 to IndicesNumber-1 do
      LocalVariantarg[ ParamNumber - iParam - (1-ParamIncrement)]
        := indices[iParam];
    if aDispIDNamed=DISPID_PROPERTYPUT then     // property_put Value
      LocalVariantArg[0] := aValue;
    Params.cArgs := ParamNumber;
    Params.rgvarg := @LocalVariantArg[0];
  end;

  // setting up named args (0 or 1 only is managed)
  Params.cNamedArgs := 0;
  Params.rgdispidNamedArgs := nil;
  if aDispIDNamed <> 0 then
  begin
    Params.cNamedArgs := 1;
    Params.rgdispidNamedArgs:= @dispidNamed;
  end;

  // Calling invoke now
  FHResult := FDispatch.Invoke( aDispID, GUID_NULL, LOCALE_SYSTEM_DEFAULT,
                             Flags, Params,
                             @varResult, @FExceptInfo, @ArgErr);

  logMessage(
    'InvokeHelper DispID="'+IntToStr(aDispID) +'", '
   +'HResult =' +IntToHex(FHResult, 12)+', '
    +'aValue = "' + VarToSTr(aValue)+'"'
   );
  CheckHResult;

end;



procedure TDispComponent.PutProperty( aDispID : Integer; anArg : TPROPVARIANT);
var
  Params: TDispParams;
  ArgErr : DWord;
  LocalVariantArg : array[0..0] of TPROPVARIANT;
  dispidNamed :  DISPID;
begin
  // Initialization
  dispidNamed := DISPID_PROPERTYPUT;
  {$HINTS OFF}
  FillChar(FExceptInfo, SizeOf(ExcepInfo),0);
  FillChar(Params, SizeOf(DispParams),0);
  FillChar(LocalVariantArg,SizeOf(LocalVariantArg),0);
  {$HINTS ON}

  LocalVariantArg[0] := anArg;
  Params.cArgs := 1;
  Params.cNamedArgs := 1;
  Params.rgdispidNamedArgs:= @dispidNamed;
  Params.rgvarg := @LocalVariantArg[0];

  FHResult := FDispatch.Invoke( aDispID, GUID_NULL, LOCALE_SYSTEM_DEFAULT,
                             DISPATCH_PROPERTYPUT, Params,
                             nil, @FExceptInfo, @ArgErr);

  logMessage('PutProperty DispID="'+IntToStr(aDispID)+'", HResult =' +IntToStr(FHResult));
  CheckHResult;

end;

procedure TDispComponent.PutProperty(aDispID: Integer; anArg: OleVariant);
var
  Params: TDispParams;
  ArgErr : DWord;
  LocalVariantArg : OleVariant;
  dispidNamed :  DISPID;
begin
  // Initialization
  dispidNamed := DISPID_PROPERTYPUT;
  {$HINTS OFF}
  FillChar(FExceptInfo, SizeOf(ExcepInfo),0);
  FillChar(Params, SizeOf(DispParams),0);
  FillChar(LocalVariantArg,SizeOf(LocalVariantArg),0);
  {$HINTS ON}

  LocalVariantArg := anArg;
  Params.cArgs := 1;
  Params.cNamedArgs := 1;
  Params.rgdispidNamedArgs:= @dispidNamed;
  Params.rgvarg := @LocalVariantArg;

  FHResult := FDispatch.Invoke( aDispID, GUID_NULL, LOCALE_SYSTEM_DEFAULT,
                             DISPATCH_PROPERTYPUT, Params,
                             nil, @FExceptInfo, @ArgErr);

  logMessage('PutProperty-Variant DispID="'+IntToStr(aDispID)
    +'", HResult =' +IntToStr(FHResult)+', '
    +'anArg = "' + VarToSTr(anArg) + '"'
    );
  CheckHResult;

end;

procedure TDispComponent.PutProperty(aDispID: Integer;
  indices: array of OleVariant; aValue: OleVariant);
var
  varResult: OleVariant;
  ArgErr : DWord;
begin
  InvokeHelper( aDispid, DISPID_PROPERTYPUT, DISPATCH_PROPERTYPUT, indices,
                aValue, ArgErr, varResult);
end;

procedure TDispComponent.SetProperty(aName: String; aValue: Boolean);
var
  propDispId : Integer;
  propArg : TPROPVARIANT;
begin
  propDispId := GetIdsOfNames( aName);
  propArg := BoolToVar( aValue);
  PutProperty( propDispId, propArg);
end;

procedure TDispComponent.SetProperty(aPropName: String; aValue: OleVariant);
var
  propDispId : Integer;
begin
  propDispId := GetIdsOfNames( aPropName);
  PutProperty( propDispId, aValue);
end;

procedure TDispComponent.SetProperty(aPropName: String;
  indices: array of OleVariant; aValue: OleVariant);
var
  propDispId : Integer;
begin
  propDispId := GetIdsOfNames( aPropName);
  PutProperty( propDispId, indices, aValue);
end;

function TDispComponent.GetProperty(aName: String): OleVariant;
var
  propDispId : Integer;
begin
  propDispId := GetIdsOfNames( aName);
  result := GetProperty(propDispId);
end;

function TDispComponent.GetProperty(aDispID: Integer): OleVariant;
var
  Params: TDispParams;
  ArgErr : DWord;
  vrResult : OleVariant;

begin

  // Initialization
  {$HINTS OFF}
  FillChar(FExceptInfo, SizeOf(ExcepInfo),0);
  FillChar(Params, SizeOf(DispParams),0);
  {$HINTS ON}

  Params.cArgs := 0;
  Params.cNamedArgs := 0;
  Params.rgdispidNamedArgs:= nil;
  Params.rgvarg := nil;

  FHResult := FDispatch.Invoke( aDispID, GUID_NULL, LOCALE_SYSTEM_DEFAULT,
                             DISPATCH_PROPERTYGET, Params,
                             @vrResult, @FExceptInfo, @ArgErr);
  logMessage('Function GetProperty DispID="'+IntToStr(aDispID)+'", HResult =' +IntToStr(FHResult));
  CheckHResult;

  result := vrResult;
end;


procedure TDispComponent.GetProperty(aName: String; out aValue: IDispatch);
var
  propDispId : Integer;
begin
  propDispId := GetIdsOfNames( aName);
  GetProperty( propDispId, aValue);
end;

procedure TDispComponent.GetProperty( aDispID: Integer;
                                      out aValue: IDispatch);
var
  Params: TDispParams;
  ArgErr : DWord;
  vrResult : TPropVariant;
begin
  aValue := nil;

  // Initialization
  {$HINTS OFF}
  FillChar(FExceptInfo, SizeOf(ExcepInfo),0);
  FillChar(Params, SizeOf(DispParams),0);
  {$HINTS ON}

  Params.cArgs := 0;
  Params.cNamedArgs := 0;
  Params.rgdispidNamedArgs:= nil;
  Params.rgvarg := nil;

  FHResult := FDispatch.Invoke( aDispID, GUID_NULL, LOCALE_SYSTEM_DEFAULT,
                             DISPATCH_PROPERTYGET, Params,
                             @vrResult, @FExceptInfo, @ArgErr);
  logMessage('GetProperty DispID="'+IntToStr(aDispID)+'", HResult =' +IntToStr(FHResult));
  CheckHResult;

  case vrResult.vt of
    VT_EMPTY, VT_NULL:
      begin {do nothing} end;
    VT_DISPATCH:
      aValue := IDispatch(vrResult.pdispVal);
    else
      raise Exception.Create( 'unknown and unhandled result from GetProperty')
  end;


end;

procedure TDispComponent.GetProperty(aName: String; index: integer;
  out aValue: IDispatch);
var
  propDispId : Integer;
begin
  propDispId := GetIdsOfNames( aName);
  GetProperty( propDispId, index, aValue);
end;

procedure TDispComponent.GetProperty(aDispID: Integer; index: integer;
  out aValue: IDispatch);
var
  Params: TDispParams;
  ArgErr : DWord;
  vrResult : TPropVariant;
  localVarArray: TPropVariant;
begin
  aValue := nil;

  // Initialization
  {$HINTS OFF}
  FillChar(FExceptInfo, SizeOf(ExcepInfo),0);
  FillChar(Params, SizeOf(DispParams),0);
  {$HINTS ON}


  localVarArray.vt := VT_INT;
  localVarArray.intVal := index;
  Params.cArgs := 1;
  Params.cNamedArgs := 0;
  Params.rgdispidNamedArgs:= nil;
  Params.rgvarg := @localVarArray;

  FHResult := FDispatch.Invoke( aDispID, GUID_NULL, LOCALE_SYSTEM_DEFAULT,
                             DISPATCH_PROPERTYGET, Params,
                             @vrResult, @FExceptInfo, @ArgErr);
  logMessage( 'GetPropertyIndex, DispID="'+IntToStr(aDispID)
    +'", Index = ' + IntToStr(Index)
    +', HResult =' +IntToStr(FHResult));
  CheckHResult;

  case vrResult.vt of
    VT_EMPTY, VT_NULL:
      begin {do nothing} end;
    VT_DISPATCH:
      aValue := IDispatch(vrResult.pdispVal);
    else
      raise Exception.Create( 'unknown and unhandled result from GetProperty')
  end;
end;

procedure TDispComponent.GetProperty(aName: String; anIndex: Variant;
  out aValue: IDispatch);
var
  propDispId : Integer;
begin
  propDispId := GetIdsOfNames( aName);
  GetProperty( propDispId, anIndex, aValue);
end;

{ defined outside procedure for long life}
procedure TDispComponent.GetProperty(aDispID: Integer; anIndex: Variant;
  out aValue: IDispatch);
var
  Params: TDispParams;
  vrResult : TPropVariant;
  localVarArray: OleVariant;
begin
  aValue := nil;
  {$HINTS OFF}
  FillChar(Params, SizeOf(DispParams),0);
  {$HINTS ON}

  localVarArray := anIndex;
  Params.cArgs := 1;
  Params.cNamedArgs := 0;
  Params.rgdispidNamedArgs:= nil;
  Params.rgvarg := @localVarArray;

  FHResult := FDispatch.Invoke( aDispID, GUID_NULL, LOCALE_SYSTEM_DEFAULT,
                             DISPATCH_PROPERTYGET, Params,
                             @vrResult, nil, nil);
 { logMessage( 'GetPropertyIndex, DispID="'+IntToStr(aDispID)
    +'", Index = ' + anIndex
    +', HResult =' +IntToHex(FHResult, 16)
    );
  CheckHResult;
  }
  case vrResult.vt of
    VT_EMPTY, VT_NULL:
      begin {do nothing} end;
    VT_DISPATCH:
      aValue := IDispatch(vrResult.pdispVal);
    else
      raise Exception.Create( 'unknown and unhandled result from GetProperty')
  end;
end;

procedure TDispComponent.GetProperty(aName: String; i1, i2: integer;
  out aValue: IDispatch);
var
  propDispId : Integer;
begin
  propDispId := GetIdsOfNames( aName);
  GetProperty( propDispId, i1, i2, aValue);
end;

procedure TDispComponent.GetProperty(aDispID: Integer; i1, i2: integer;
  out aValue: IDispatch);
var
  Params: TDispParams;
  ArgErr : DWord;
  vrResult : TPropVariant;
  localVarArray: array[0..1] of OleVariant;
begin
  aValue := nil;

  // Initialization
  {$HINTS OFF}
  FillChar(FExceptInfo, SizeOf(ExcepInfo),0);
  FillChar(Params, SizeOf(DispParams),0);
  {$HINTS ON}

  // arguments are inversed
  //localVarArray[0].vt := VT_I2;
  //localVarArray[0].intVal := i2;
  //localVarArray[1].vt := VT_I2;
  //localVarArray[1].intVal := i1;
  localVarArray[0] := i2;
  localVarArray[1] := i1;
  Params.cArgs := 2;
  Params.cNamedArgs := 0;
  Params.rgdispidNamedArgs:= nil;
  Params.rgvarg := @localVarArray;

  FHResult := FDispatch.Invoke( aDispID, GUID_NULL, LOCALE_SYSTEM_DEFAULT,
                             DISPATCH_PROPERTYGET, Params,
                             @vrResult, @FExceptInfo, @ArgErr);

  // We get a SIGSEV error here: access violation on VarR4FromCy incorrectly called.
  logMessage( 'GetPropertyIndex, DispID="'+IntToStr(aDispID)
    +'", (I1, I2) = (' + IntToStr(i1)+', '+ IntToStr(i2)+')'
    +', HResult =' +IntToStr(FHResult));
  CheckHResult;

  case vrResult.vt of
    VT_EMPTY, VT_NULL:
      begin {do nothing} end;
    VT_DISPATCH:
      aValue := IDispatch(vrResult.pdispVal);
    else
      raise Exception.Create( 'unknown and unhandled result from GetProperty')
  end;
end;

procedure TDispComponent.CallProcedure(aName: String);
var
  propDispId : Integer;
begin
  propDispId := GetIdsOfNames( aName);
  CallProcedure( propDispId);
end;

procedure TDispComponent.CallProcedure(aDispid: Integer);
var
  Params: TDispParams;
begin
  {$HINTS OFF}
  FillChar(Params, SizeOf(DispParams),0);
  {$HINTS ON}
  //Params.cArgs             := 0;
  //Params.cNamedArgs        := 0;
  //Params.rgvarg            := nil;
  //Params.rgdispidNamedArgs := nil;
  FHResult := FDispatch.Invoke( aDispid, GUID_NULL, LOCALE_SYSTEM_DEFAULT,
                                DISPATCH_METHOD, Params,
                                nil, nil, nil);
  logMessage('CallProcedure DispID="'+IntToStr(aDispID)+'", HResult =' +IntToStr(FHResult));
  CheckHResult;
end;

procedure TDispComponent.CallProcedure(aName: String; args: array of OleVariant
  );
var
  propDispId : Integer;
begin
  propDispId := GetIdsOfNames( aName);
  CallProcedure( propDispId, args);
end;

procedure TDispComponent.CallProcedure(aDispid: Integer;
  args: array of OleVariant);
var
  varResult: OleVariant;
  ArgErr : DWord;
begin
  InvokeHelper( aDispid, 0, DISPATCH_METHOD, args,
                Null, ArgErr, varResult);
end;

function TDispComponent.CallFunctionDisp(aName: String): IDispatch;
var
  propDispId : Integer;
begin
  propDispId := GetIdsOfNames( aName);
  result := CallFunctionDisp( propDispId);
end;

function TDispComponent.CallFunctionDisp(aDispid: Integer): IDispatch;
var
  Params: TDispParams;
  vrResult : TPropVariant;
begin
  Params.cArgs             := 0;
  Params.cNamedArgs        := 0;
  Params.rgvarg            := nil;
  Params.rgdispidNamedArgs := nil;
  FHResult := FDispatch.Invoke( aDispid, GUID_NULL, LOCALE_SYSTEM_DEFAULT,
                                DISPATCH_METHOD, Params,
                                @vrResult, nil, nil);
  logMessage('CallFunctionDisp DispID="'+IntToStr(aDispID)+'", HResult =' +IntToStr(FHResult));
  CheckHResult;

  case vrResult.vt of
    VT_EMPTY, VT_NULL:
      begin {do nothing} end;
    VT_DISPATCH:
      result := IDispatch(vrResult.pdispVal);
    else
      raise Exception.Create( 'unknown and unhandled result from GetProperty')
  end;
end;

function TDispComponent.CallFunctionDisp(aDispid: Integer;
  args: array of OleVariant): IDispatch;
var
  varResult: OleVariant;
  ArgErr : DWord;
begin
  result := nil;
  InvokeHelper( aDispid, 0, DISPATCH_METHOD, args,
                null, ArgErr, varResult);
  result := varResult;
end;

constructor TDispComponent.Create(anOwner: TComponent);
begin
  inherited Create(anOwner);
end;

constructor TDispComponent.Create(anOwner: TComponent; aDispatch: IDispatch);
begin
  inherited Create( anOwner);
  FDispatch := aDispatch;
end;

{ TDispCollection }

function TDispCollection.GetItem(index: OleVariant): TDispComponent;
var
  aVar :IDispatch;
begin
  GetProperty(DISPID_ITEM, index, aVar);
  result := ItemClassType.Create(self, aVar);
end;

function TDispCollection.Count: Integer;
begin
  result := GetProperty(DISPID_COUNT);
end;

{ TDispServer }

procedure TDispServer.SetServerName(const AValue: String);
begin
  if FServerName=AValue then exit;
  FServerName:=AValue;
end;

procedure TDispServer.SetVisible(const AValue: Boolean);
begin
  if FVisible=AValue then exit;
  PutProperty( DISPID_VISIBLE, aValue);
  FVisible:=AValue;
end;

procedure TDispServer.CreateComServer;
var
  GuID : TGUID;
begin
  //Preconditions:
  If FDispatch <> nil then exit;

  FDispatch := nil;
  GuID := ProgIDToClassID(ServerName);
  FHResult := CoCreateInstance(GuiD,nil,CLSCTX_INPROC_SERVER or CLSCTX_LOCAL_SERVER, IDispatch, FDispatch);
  CheckHResult;
end;

procedure TDispServer.SetActive(const AValue: Boolean);
begin
  if FActive=AValue then exit;

  if AValue then
    CreateComServer
  else
    Quit;

  FActive:=AValue;
end;

constructor TDispServer.Create(anOwner: TComponent);
begin
  inherited Create(anOwner);
end;

destructor TDispServer.Destroy;
begin
  if Active then Quit;
  inherited Destroy;
end;

procedure TDispServer.Quit;
begin
  if Not Active then Exit;
  FActive := false;
  CallProcedure( DISPID_QUIT);
end;

end.

