{******************************************************************************}
{                                                                              }
{ Excel Interface Component                                                    }
{ Component ExcelUtilities.pas                                                 }
{ This interface component can be downloaded at www.tcoq.org, component page.  }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License Version  }
{ 1.1 (the "License"); you may not use this file except in compliance with the }
{ License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ The Original Code is ExcelUtilities.pas                                      }
{                                                                              }
{ The Initial Developer of the Original Code is Thierry.Coq                    }
{ thierry.coq(at)centraliens.net                                               }
{ Portions created by this individual are Copyright (C)                        }
{ 2000-2008 of this individual.                                                }
{                                                                              }
{ Last modified: 2009/07/25                                                    }
{ Version 0.4                                                                  }
{                                                                              }
{******************************************************************************}
unit ExcelUtilities;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Windows, Variants, ComObj, ActiveX,
  Diagnostics, ComUtilities;

// constants for excel functions and properties.
// See Excel Reference Manual for usage.
{$INCLUDE 'ExcelConstants.inc'}

type
  TLineDashStyle = (lsContinuous = xlContinuous,
    lsDashDot       = xlDashDot,
    lsDashDotDot    = xlDashDotDot,
    lsDot           = xlDot,
    lsDouble        = xlDouble,
    lsLineStyleNone = xlLineStyleNone,
    lsSlantDashDot  = xlSlantDashDot,
    lsDash          = xlDash
    );

  TMSOLineDashStyle = (
    msoLineDashStyleMixed  = -2,
    msoLineSolid           = 1,
    msoLineSquareDot       = 2,
    msoLineRoundDot        = 3,
    msoLineDash            = 4,
    msoLineDashDot         = 5,
    msoLineDashDotDot      = 6
    );


  TBorderWeight = (
    bwHairline = xlHairline,
    bwMedium   = xlMedium,
    bwThick    = xlThick,
    bwThin     = xlThin //default
    );

  TBorderIndex = (
    xlDiagonalDown     = 5,
    xlDiagonalUp       = 6,
    xlEdgeLeft         = 7,
    xlEdgeTop          = 8,
    xlEdgeBottom       = 9,
    xlEdgeRight        = 10,
    xlInsideVertical   = 11,
    xlInsideHorizontal = 12
    );



  // TRGBColor: Color codes:
  // - $FF0000 is Blue
  // - $00FF00 is Green
  // - $0000FF is Red
  // - use a combination for other colors.
  TRGBColor = Longword;

  THorizontalAlignment = (
    HAlignCenter = xlHAlignCenter,
    HAlignCenterAcrossSelection = xlHAlignCenterAcrossSelection,
    HAlignDistributed = xlHAlignDistributed,
    HAlignFill = xlHAlignFill,
    HAlignGeneral = xlHAlignGeneral,
    HAlignJustify = xlHAlignJustify,
    HAlignLeft = xlHAlignLeft,
    HAlignRight = xlHAlignRight
  );

  TVerticalAlignment = (
    VAlignTop = xlVAlignTop,
    VAlignJustify = xlVAlignJustify,
    VAlignDistributed = xlVAlignDistributed,
    VAlignCenter = xlVAlignCenter,
    VAlignBottom = xlVAlignBottom
    );

  // implements MsoAutoShapeType
  TAutoShapeType = (
    msoShapeRectangle        =  1,
    msoShapeDiamond          =  4,
    msoShapeOval             =  9,
    msoShapeHexagon          = 10,
    msoShapeArc              = 25,

    msoShapeFlowchartProcess                  = 61,
    msoShapeFlowchartAlternateProcess         = 62,
    msoShapeFlowchartDecision                 = 63,
    msoShapeFlowchartData                     = 64,
    msoShapeFlowchartPredefinedProcess        = 65,
    msoShapeFlowchartInternalStorage          = 66,
    msoShapeFlowchartDocument                 = 67,
    msoShapeFlowchartMultidocument            = 68,
    msoShapeFlowchartTerminator               = 69,
    msoShapeFlowchartPreparation              = 70,
    msoShapeFlowchartManualInput              = 71,
    msoShapeFlowchartManualOperation          = 72,
    msoShapeFlowchartConnector                = 73,
    msoShapeFlowchartOffpageConnector         = 74,
    msoShapeFlowchartCard                     = 75,
    msoShapeFlowchartPunchedTape              = 76,
    msoShapeFlowchartSummingJunction          = 77,
    msoShapeFlowchartOr                       = 78,
    msoShapeFlowchartCollate                  = 79,
    msoShapeFlowchartSort                     = 80,
    msoShapeFlowchartExtract                  = 81,
    msoShapeFlowchartMerge                    = 82,
    msoShapeFlowchartStoredData               = 83,
    msoShapeFlowchartDelay                    = 84,
    msoShapeFlowchartSequentialAccessStorage  = 85,
    msoShapeFlowchartMagneticDisk             = 86,
    msoShapeFlowchartDirectAccessStorage      = 87,
    msoShapeFlowchartDisplay                  = 88

    {TODO 2 -oTC -cCompletion : many more shape types to be added, see Excel help file and below}
    {msoShape16pointStar
    msoShape24pointStar
    msoShape32pointStar
    msoShape4pointStar
    msoShape5pointStar
    msoShape8pointStar
    msoShapeActionButtonBackorPrevious
    msoShapeActionButtonBeginning
    msoShapeActionButtonCustom
    msoShapeActionButtonDocument
    msoShapeActionButtonEnd
    msoShapeActionButtonForwardorNext
    msoShapeActionButtonHelp
    msoShapeActionButtonHome
    msoShapeActionButtonInformation
    msoShapeActionButtonMovie
    msoShapeActionButtonReturn
    msoShapeActionButtonSound
    msoShapeArc
    msoShapeBalloon
    msoShapeBentArrow
    msoShapeBentUpArrow
    msoShapeBevel
    msoShapeBlockArc
    msoShapeCan
    msoShapeChevron
    msoShapeCircularArrow
    msoShapeCloudCallout
    msoShapeCross
    msoShapeCube
    msoShapeCurvedDownArrow
    msoShapeCurvedDownRibbon
    msoShapeCurvedLeftArrow
    msoShapeCurvedRightArrow
    msoShapeCurvedUpArrow
    msoShapeCurvedUpRibbon
    msoShapeDiamond
    msoShapeDonut
    msoShapeDoubleBrace
    msoShapeDoubleBracket
    msoShapeDoubleWave
    msoShapeDownArrow
    msoShapeDownArrowCallout
    msoShapeDownRibbon
    msoShapeExplosion1
    msoShapeExplosion2
    msoShapeFoldedCorner
    msoShapeHeart
    msoShapeHexagon
    msoShapeHorizontalScroll
    msoShapeIsoscelesTriangle
    msoShapeLeftArrow
    msoShapeLeftArrowCallout
    msoShapeLeftBrace
    msoShapeLeftBracket
    msoShapeLeftRightArrow
    msoShapeLeftRightArrowCallout
    msoShapeLeftRightUpArrow
    msoShapeLeftUpArrow
    msoShapeLightningBolt
    msoShapeLineCallout1
    msoShapeLineCallout1AccentBar
    msoShapeLineCallout1BorderandAccentBar
    msoShapeLineCallout1NoBorder
    msoShapeLineCallout2
    msoShapeLineCallout2AccentBar
    msoShapeLineCallout2BorderandAccentBar
    msoShapeLineCallout2NoBorder
    msoShapeLineCallout3
    msoShapeLineCallout3AccentBar
    msoShapeLineCallout3BorderandAccentBar
    msoShapeLineCallout3NoBorder
    msoShapeLineCallout4
    msoShapeLineCallout4AccentBar
    msoShapeLineCallout4BorderandAccentBar
    msoShapeLineCallout4NoBorder
    msoShapeMixed
    msoShapeMoon
    msoShapeNoSymbol
    msoShapeNotchedRightArrow
    msoShapeNotPrimitive
    msoShapeOctagon
    msoShapeOval
    msoShapeOvalCallout
    msoShapeParallelogram
    msoShapePentagon
    msoShapePlaque
    msoShapeQuadArrow
    msoShapeQuadArrowCallout
    msoShapeRectangle
    msoShapeRectangularCallout
    msoShapeRegularPentagon
    msoShapeRightArrow
    msoShapeRightArrowCallout
    msoShapeRightBrace
    msoShapeRightBracket
    msoShapeRightTriangle
    msoShapeRoundedRectangle
    msoShapeRoundedRectangularCallout
    msoShapeSmileyFace
    msoShapeStripedRightArrow
    msoShapeSun
    msoShapeTrapezoid
    msoShapeUpArrow
    msoShapeUpArrowCallout
    msoShapeUpDownArrow
    msoShapeUpDownArrowCallout
    msoShapeUpRibbon
    msoShapeUTurnArrow
    msoShapeVerticalScroll
    msoShapeWave}
  );

  TTriState = (
    msoTrue = -1,
    msoFalse = 0
    {TODO 2 -oTC -cCompletion : other possibilities exist in msoTriState}
  );

  //TUnderlineStyle implements xlUnderLineStyle
  TUnderlineStyle = (
    xlUnderlineStyleNone             = -4142,
    xlUnderlineStyleDouble           = -4119,
    xlUnderlineStyleSingle           =     2,
    xlUnderlineStyleSingleAccounting =     4,
    xlUnderlineStyleDoubleAccounting =     5
  );

  // implements MsoGradientStyle
  TGradientStyle = (
    msoGradientMixed        = -2,
    msoGradientHorizontal   = 1,
    msoGradientVertical     = 2,
    msoGradientDiagonalUp   = 3,
    msoGradientDiagonalDown = 4,
    msoGradientFromCorner   = 5,
    msoGradientFromTitle    = 6,
    msoGradientFromCenter   = 7
  );

  TLineStyle = (
    msoLineStyleMixed       = -2,
    msoLineSingle           = 1,
    msoLineThinThin         = 2,
    msoLineThinThick        = 3,
    msoLineThickThin        = 4,
    msoLineThickBetweenThin = 5
  );

  TConnectorType = (
    msoConnectorTypeMixed = -2,
    msoConnectorStraight  = 1,
    msoConnectorElbow     = 2,
    msoConnectorCurve     = 3
  );

  TSheetType = (
    xlChart                = $FFFFEFF3,
    xlDialogSheet          = $FFFFEFEC,
    xlExcel4MacroSheet     = $00000003,
    xlExcel4IntlMacroSheet = $00000004,
    xlWorksheet            = $FFFFEFB9

  );

  // a few chart types
  TChartType = (
    xlLine         =     4,
    xlPie          =     5,
    xlRadar        = -4151,
    xlXYScatter    = $FFFFEFB7  //chart type

  {
  xlLineMarkersStacked. Stacked Line with Markers
  xlLineStacked. Stacked Line
  xlPieOfPie. Pie of Pie
  xlPyramidBarStacked. Stacked Pyramid Bar
  xlPyramidCol. 3D Pyramid Column
  xlPyramidColClustered. Clustered Pyramid Column
  xlPyramidColStacked. Stacked Pyramid Column
  xlPyramidColStacked100. 100% Stacked Pyramid Column
  xlRadarFilled. Filled Radar
  xlRadarMarkers. Radar with Data Markers
  xlStockHLC. High-Low-Close
  xlStockOHLC. Open-High-Low-Close
  xlStockVHLC. Volume-High-Low-Close
  xlStockVOHLC. Volume-Open-High-Low-Close
  xlSurface. 3D Surface
  xlSurfaceTopView. Surface (Top View)
  xlSurfaceTopViewWireframe. Surface (Top View wireframe)
  xlSurfaceWireframe. 3D Surface (wireframe)
  xlXYScatter. Scatter
  xlXYScatterLines. Scatter with Lines.
  xlXYScatterLinesNoMarkers. Scatter with Lines and No Data Markers
  xlXYScatterSmooth. Scatter with Smoothed Lines
  xlXYScatterSmoothNoMarkers. Scatter with Smoothed Lines and No Data Markers
  xl3DArea. 3D Area
  xl3DAreaStacked. 3D Stacked Area
  xl3DAreaStacked100. 100% Stacked Area
  xl3DBarClustered. 3D Clustered Bar
  xl3DBarStacked. 3D Stacked Bar
  xl3DBarStacked100. 3D 100% Stacked Bar
  xl3DColumn. 3D Column
  xl3DColumnClustered. 3D Clustered Column
  xl3DColumnStacked. 3D Stacked Column
  xl3DColumnStacked100. 3D 100% Stacked Column
  xl3DLine. 3D Line
  xl3DPie. 3D Pie
  xl3DPieExploded. Exploded 3D Pie
  xlArea. Area
  xlAreaStacked. Stacked Area
  xlAreaStacked100. 100% Stacked Area
  xlBarClustered. Clustered Bar
  xlBarOfPie. Bar of Pie
  xlBarStacked. Stacked Bar
  xlBarStacked100. 100% Stacked Bar
  xlBubble. Bubble
  xlBubble3DEffect. Bubble with 3D effects
  xlColumnClustered. Clustered Column
  xlColumnStacked. Stacked Column
  xlColumnStacked100. 100% Stacked Column
  xlConeBarClustered. Clustered Cone Bar
  xlConeBarStacked. Stacked Cone Bar
  xlConeBarStacked100. 100% Stacked Cone Bar
  xlConeCol. 3D Cone Column
  xlConeColClustered. Clustered Cone Column
  xlConeColStacked. Stacked Cone Column
  xlConeColStacked100. 100% Stacked Cone Column
  xlCylinderBarClustered. Clustered Cylinder Bar
  xlCylinderBarStacked. Stacked Cylinder Bar
  xlCylinderBarStacked100. 100% Stacked Cylinder Bar
  xlCylinderCol. 3D Cylinder Column
  xlCylinderColClustered. Clustered Cone Column
  xlCylinderColStacked. Stacked Cone Column
  xlCylinderColStacked100. 100% Stacked Cylinder Column
  xlDoughnut. Doughnut
  xlDoughnutExploded. Exploded Doughnut
  xlLineMarkers. Line with Markers
  xlLineMarkersStacked100. 100% Stacked Line with Markers
  xlLineStacked100. 100% Stacked Line
  xlPieExploded. Exploded Pie
  xlPyramidBarClustered. Clustered Pyramid Bar
  xlPyramidBarStacked100. 100% Stacked Pyramid Bar
  }
  );

  TPlotBy = (
    xlRows = $00000001,
    xlColumns = $00000002
  );


const
  //Special dispid values:
  DISPID_DEFAULT             =    0; //default or Value dispid is zero.

  //Standard dispid values:
  DISPID_ADD                 =  181;
  DISPID_ADDCONNECTOR        = 1714;
  DISPID_ADDLINE             = 1722;
  DISPID_ADDSHAPE            = 1727;
  //DISPID_ADDCOMMENT        = ?? Unknown
  DISPID_AUTOSIZE            =  614;
  DISPID_BACKCOLOR           =  100;
  DISPID_BEGINCONNECT        = 1750;
  DISPID_BEGINDISCONNECT     = 1753;
  DISPID_BOLD                =   96;
  DISPID_BORDER              =  128;
  DISPID_BORDERAROUND        = 1067;
  DISPID_BORDERS             =  435;
  DISPID_CELLS               =  238;
  DISPID_CHARACTERS          =  603;
  DISPID_CHARTS              =  121;
  DISPID_CHARTTITLE          =   81; //?
  DISPID_CHARTTYPE           =  108;
  DISPID_CLOSE               =  277;
  DISPID_COLOR               =   99;
  DISPID_COLUMN              =  240;
  DISPID_COLUMNWIDTH         =  242;
  DISPID_COLUMNS             =  241;
  DISPID_COMMENT             =  910;
  DISPID_COMMENTS            =  575;
  DISPID_CONNECTOR           = 1696;
  DISPID_CONNECTORFORMAT     = 1697;
  DISPID_COPY                =  551;
  DISPID_DELETE              =  117;
  DISPID_DISPLAYALERTS       =  343;
  DISPID_END                 =  500;
  DISPID_ENDCONNECT          = 1754;
  DISPID_ENDDISCONNECT       = 1755;
  DISPID_FILL                = 1663;
  DISPID_FONT                =  146;
  DISPID_FONTSTYLE           =  177;
  DISPID_FORECOLOR           =  101;
  DISPID_FORECOLOR_LINE      =  108;
  DISPID_FORMULA             =  261;
  DISPID_HEIGHT              =  123;
  DISPID_HORIZONTALALIGNMENT =  136;
  DISPID_INTERIOR            =  129;
  DISPID_ITALIC              =  101;
  DISPID_LEFT                =  127;
  DISPID_LEGEND              =   84;
  DISPID_LEGENDENTRIES       =  173;
  DISPID_LINE                =  817;
  DISPID_LINEDASHSTYLE       =  104;
  DISPID_LINESTYLE           =  110;
  DISPID_NAME                =  110;
  DISPID_NUMBERFORMAT        =  193;
  DISPID_PASTE               =  211;
  DISPID_RANGE               =  197;
  DISPID_REPLACE             =  226;
  DISPID_REROUTECONNECTIONS  = 1683;
  DISPID_RGB                 = DISPID_DEFAULT; //RGB is default value for ColorFormat.
  DISPID_ROW                 =  257;
  DISPID_ROWHEIGHT           =  272;
  DISPID_ROWS                =  258;
  DISPID_SAVE                =  283;
  DISPID_SAVEAS              = 1925;
  DISPID_SCHEMECOLOR         =  100;
  DISPID_SELECT              =  235;
  DISPID_SELECTION           =  147;
  DISPID_SETSOURCEDATA       = 1413;
  DISPID_SHAPES              = 1377;
  DISPID_SHADOW              =  103;
  DISPID_SHEETS              =  485;
  DISPID_SIZE                =  104;
  DISPID_SOLID               =   15;
  DISPID_SUBSCRIPT           =  179;
  DISPID_SUPERSCRIPT         =  178;
  DISPID_TEXT                =  138;
  DISPID_TEXTFRAME           = 1692;
  DISPID_TRANSPARENCY        =  111;
  DISPID_TOP                 =  126;
  DISPID_TYPE                =  108;
  DISPID_TWOCOLORGRADIENT    =   16;
  DISPID_UNDERLINESTYLE      =  106;
  DISPID_USERPICTURE         =   17;
  DISPID_VALUE               =    6;
  DISPID_VERTICALALIGNMENT   =  137;
  DISPID_VISIBLE_FILL        =  113; //special value for FillFormat
  DISPID_WEIGHT              =  113;
  DISPID_WIDTH               =  122;
  DISPID_WORKBOOKS           =  572;
  DISPID_WRAPTEXT            =  276;

// type
//   _Global = Excel.Application;
//     function Workbooks(Index: OleVariant): OleVariant; dispid 572;

type


  { All components written for Excel should inherit from TExcelBaseComponent,
    and not directly from TDispComponent.                                      }

  { TExcelBaseComponent }

  TExcelBaseComponent = class(TDispComponent)
  public
    procedure Delete;
  end;

  { TExcelComponent }
  //provides many standard excel properties, be careful, might not be valid
  //always
  TExcelComponent = class(TExcelBaseComponent)
  private
  protected
    function GetVisible: Boolean; virtual;
    procedure SetVisible(const AValue: Boolean); virtual;
  published
    property Visible: Boolean read GetVisible write SetVisible;
  end;

  { Various Excel Objects needed for coding}
  {============================================================================}

  { TInterior }
  TInterior = class( TExcelBaseComponent)
  private
    function GetColor: TRGBColor;
    procedure SetColor(const AValue: TRGBColor);
  public
    property Color: TRGBColor read GetColor write SetColor;
  end;

  { TFont }
  TFont = class( TExcelBaseComponent)
  private
    function GetBold: Boolean;
    function GetColor: TRGBColor;
    function GetFontName: String;
    function GetFontStyle: String;
    function GetItalic: Boolean;
    function GetShadow: Boolean;
    function GetSize: Integer;
    function GetSubscript: Boolean;
    function GetSuperscript: Boolean;
    function GetUnderlineStyle: TUnderlineStyle;
    procedure SetBold(const AValue: Boolean);
    procedure SetColor(const AValue: TRGBColor);
    procedure SetFontName(const AValue: String);
    procedure SetFontStyle(const AValue: String);
    procedure SetItalic(const AValue: Boolean);
    procedure SetShadow(const AValue: Boolean);
    procedure SetSize(const AValue: Integer);
    procedure SetSubscript(const AValue: Boolean);
    procedure SetSuperscript(const AValue: Boolean);
    procedure SetUnderlineStyle(const AValue: TUnderlineStyle);
  published
    { TODO 2 -oTC -cCompletion : need to replace integer by TColor in Set/Get Color
of "xlEdgeBottom"?
       - Underline: xlUnderLineStyle
    }
    // Color requires a RGB value, see the RGB function in Excel reference manual
    property Bold: Boolean read GetBold write SetBold;
    property Color : TRGBColor read GetColor write SetColor;
    property FontName: String read GetFontName write SetFontName;
    property FontStyle: String read GetFontStyle write SetFontStyle;
    property Italic: Boolean read GetItalic write SetItalic;
    property Size: Integer read GetSize write SetSize;
    property Subscript: Boolean  read GetSubscript write SetSubscript;
    property Superscript: Boolean  read GetSuperscript write SetSuperscript;
    property Shadow: Boolean  read GetShadow write SetShadow;
    property Underline: TUnderlineStyle  read GetUnderlineStyle write SetUnderlineStyle;
  end;

  { TCharacters }

  TCharacters = class(TExcelBaseComponent)
  private
    function GetText: String;
    procedure SetText(const AValue: String);
    protected
    public
      function Font: TFont;
    published
      property Text: String read GetText write SetText;

  end;

  { TTextFrame }

  TTextFrame = class(TExcelBaseComponent)
  private
    function GetAutosize: Boolean;
    procedure SetAutosize(const AValue: Boolean);
  public
    function Characters: TCharacters;
    property Autosize: Boolean read GetAutosize write SetAutosize;
  end;

  { TColorFormat }

  TColorFormat = class(TExcelBaseComponent)
  private
    function GetRGB: TRGBColor;
    function GetSchemeColor: Integer;
    procedure SetRGB(const AValue: TRGBColor);
    procedure SetSchemeColor(const AValue: Integer);
  published
    property RGB: TRGBColor read GetRGB write SetRGB;
    property SchemeColor : Integer read GetSchemeColor write SetSchemeColor;
  end;

  { TXlsCustomShape }
  TXlsCustomShape = class(TExcelBaseComponent)
  protected
    function GetVisible: TTriState; virtual;
    procedure SetVisible(const AValue: TTriState); virtual;
  published
    property Visible: TTriState read GetVisible write SetVisible;
  end;

  { TXlsCustomFormat }
  TXlsCustomFormat = class(TXlsCustomShape)
  private
    function GetBackColor: TColorFormat;
    function GetTransparency: Double;
    procedure SetTransparency(const AValue: Double);
  protected
  public
    function GetForeColor: TColorFormat; virtual;
  published
    property Transparency: Double read GetTransparency write SetTransparency;
    property ForeColor: TColorFormat read GetForeColor;
    property BackColor: TColorFormat read GetBackColor;
  end;

  { TFillFormat }
  TFillFormat = class(TXlsCustomFormat)
  public
    procedure Solid;
    procedure TwoColorGradient( aGradientStyle:TGradientStyle; aVariant: Integer);
    procedure UserPicture( aPictureFileName:String);
  end;

  { TLineFormat }
  TLineFormat = class(TXlsCustomFormat)
  private
    function GetDashStyle: TMSOLineDashStyle;
    function GetStyle: TLineStyle;
    function GetWeight: Single;
    procedure SetDashStyle(const AValue: TMSOLineDashStyle);
    procedure SetStyle(const AValue: TLineStyle);
    procedure SetWeight(const AValue: Single);
  public
    //The line's forecolor has a different dispid!?
    function GetForeColor: TColorFormat; override;
  published
    property DashStyle: TMSOLineDashStyle read GetDashStyle write SetDashStyle;
    property Style:TLineStyle  read GetStyle write SetStyle;
    property Weight: Single  read GetWeight write SetWeight;
  end;

  TShape = class;
  { TConnectorFormat }
  TConnectorFormat = class( TExcelBaseComponent)
  public
     procedure BeginConnect( aShape:TShape; aConnectionSite: Long);
     procedure EndConnect( aShape:TShape; aConnectionSite: Long);
     procedure BeginDisconnect;
     procedure EndDisconnect;
  end;

  { TShape }
  TShape = class( TXlsCustomShape)
    private
      function GetHeight: Integer;
      function GetLeft: Integer;
      function GetTop: Integer;
      function GetWidth: Integer;
      procedure SetHeight(const AValue: Integer);
      procedure SetLeft(const AValue: Integer);
      procedure SetTop(const AValue: Integer);
      procedure SetWidth(const AValue: Integer);
    public
      function Fill: TFillFormat;
      function Line: TLineFormat;
      function TextFrame: TTextFrame;
      function ConnectorFormat: TConnectorFormat;
      function Connector: TTriState;
      procedure RerouteConnections;
      property Height: Integer read GetHeight write SetHeight;
      property Width: Integer read GetWidth write SetWidth;
      property Left: Integer read GetLeft write SetLeft;
      property Top: Integer read GetTop write SetTop;
  end;

  { TShapes }
  TShapes = class( TDispCollection)
  public
    Constructor Create( anOwner : TComponent); override; overload;
    Constructor Create( anOwner : TComponent; aDispatch : IDispatch); overload;
    function GetItem( index: OleVariant): TShape; override;
    function AddConnector( AType: TConnectorType; BeginX, BeginY, Width, Height : Single): TShape;
    function AddLine( BeginX, BeginY, EndX, Endy : Single): TShape;
    function AddShape( aShapeType:TAutoShapeType; left, top, width, height : Single): TShape;
  end;


  { TComment }
  TComment = class( TExcelComponent)
{ TODO 2 -oTC -cCompletion : properties to add
    - Text (method)
    - Shape
    - Delete
    }
  public
    procedure Delete;
  published
  end;

  { TBorder }

  TBorder = class( TExcelBaseComponent)
  private
    function GetLineStyle: TLineDashStyle;
    procedure SetLineStyle(const AValue: TLineDashStyle);
  published
  (* TODO 3 -oTC -cBug : properties to add, when borders bug is corrected.
    - property Color: OleVariant dispid 99;
    - property ColorIndex: OleVariant dispid 97;
    - property LineStyle: OleVariant dispid 119;
    - property Weight: OleVariant dispid 120;
    *)
    property LineStyle: TLineDashStyle read GetLineStyle write SetLineStyle;
  end;

  { TBorders }
  TBorders = class(TDispCollection)
    (* TODO 3 -oTC -cBug : Can't find the Border object? Why? *)
    Constructor Create( anOwner : TComponent); override; overload;
    Constructor Create( anOwner : TComponent; aDispatch : IDispatch); overload;
    function GetItem( index: OleVariant): TBorder; override;
  end;


  { TRange }
  TRange = class( TExcelBaseComponent)
  private
    function GetBorders( index: TBorderIndex): TBorder;
    function GetColumn: Long;
    function GetColumns(index: integer): TRange;
    function GetColumnWidth: Single;
    function GetComment: TComment;
    function GetFont: TFont;
    function GetFormula: String;
    function GetHeight: Long;
    function GetHorizontalAlignment: THorizontalAlignment;
    function GetInterior: TInterior;
    function GetNumberFormat: String;
    function GetRow: Long;
    function GetRowHeight: Single;
    function GetRows(index: integer): TRange;
    function GetText: String;
    function GetValue: OleVariant;
    function GetVerticalAlignment: TVerticalAlignment;
    function GetWidth: Long;
    function GetWrapText: Boolean;
    procedure SetColumnWidth(const AValue: Single);
    procedure SetFormula(const AValue: String);
    procedure SetHorizontalAlignment(const AValue: THorizontalAlignment);
    procedure SetNumberFormat(const AValue: String);
    procedure SetRowHeight(const AValue: Single);
    procedure SetValue(const AValue: OleVariant);
    procedure SetColumnWidth(const AValue: Long);
    procedure SetVerticalAlignment(const AValue: TVerticalAlignment);
    procedure SetWrapText(const AValue: Boolean);
  public
    // xlUp, or xlDown
    function End_( aDirection : LongWord) : TRange;
    function GetCount: Long;
    procedure AddComment( aText:String);

    // Procedure BorderAround
    // From the Excel help:
    // - You must specify either ColorIndex or Color, but not both.
    // - You can specify either LineStyle or Weight, but not both.
    //   If you don't specify either argument, Microsoft Excel uses the default line style and weight.
    Procedure BorderAround( LineStyle: TLineDashStyle; Weight: TBorderWeight; ColorIndex: Short; Color: TRGBColor);

    procedure Copy;
    procedure Replace( What,
                       Replacement: String;
                       Lookat: long;
                       SearchOrder: long;
                       MatchCase: Boolean);

    procedure Select;
    property Rows[index:integer] : TRange read GetRows;
    property Columns[index:integer] : TRange read GetColumns;
    (* TODO 3 -oTC -cBug : Borders is not working. Why? Seems the property is not there.
       Most probably a bad call, with bad arguments.
    property Borders[ index:TBorderIndex] : TBorder read GetBorders; *)
  published
    property Column: Long read GetColumn;
    property ColumnWidth: Single read GetColumnWidth write SetColumnWidth;
    property Comment: TComment read GetComment;
    property Count: Long read GetCount;
    property Font: TFont read GetFont;
    property Formula: String read GetFormula write SetFormula;
    property Height: Long read GetHeight;
    property Interior: TInterior read GetInterior;
    property Row: Long read GetRow;
    property RowHeight: Single read GetRowHeight write SetRowHeight;
    property Text: String read GetText;
    property Value: OleVariant read GetValue write SetValue;
    property Width: Long read GetWidth;
    property WrapText: Boolean read GetWrapText write SetWrapText;
    property VerticalAlignment: TVerticalAlignment read GetVerticalAlignment
      write SetVerticalAlignment;
    property HorizontalAlignment: THorizontalAlignment read GetHorizontalAlignment
      write SetHorizontalAlignment;
    property NumberFormat: String read GetNumberFormat write SetNumberFormat;


  end;

  { TComments }
  TComments = class(TDispCollection)
  public
    Constructor Create( anOwner : TComponent); override; overload;
    Constructor Create( anOwner : TComponent; aDispatch : IDispatch); overload;
  end;

  { TAbstractSheet }
  TAbstractSheet = class(TExcelBaseComponent)
  protected
    function GetType: TSheetType;
    procedure SetType(const AValue: TSheetType);
  published
    property Type_: TSheetType read GetType write SetType;
  end;

  { TSheet }
  TSheet = class( TAbstractSheet)
  public
    function Cells(RowIndex, ColumnIndex:Integer): TRange;
    function Rows: TRange;
    function GetLastRowFilledIn( ColumnIndex: Integer): Integer;
    procedure Paste;
    function Shapes: TShapes;
    function Range(aRange:String): TRange;
    function Comments: TComments;
  end;

  { TSheets }
  TSheets = class( TDispCollection)
  public
    Constructor Create( anOwner : TComponent); override; overload;
    Constructor Create( anOwner : TComponent; aDispatch : IDispatch); overload;
    function GetItem( index: OleVariant): TSheet; override;
    function Sheet(index: OleVariant): TSheet;
  end;

  { TChartTitle }

  TChartTitle = class( TExcelBaseComponent)
  private
    function GetBorder: TBorder;
    function GetCharacters: TCharacters;
    function GetFill: TFillFormat;
    function GetInterior: TInterior;
    function GetText: String;
    procedure SetText(const AValue: String);
  public
    property Border: TBorder read GetBorder;
    property Characters: TCharacters read GetCharacters;
    property Fill: TFillFormat read GetFill;
    property Interior: TInterior read GetInterior;
    property Text: String read GetText write SetText;
  end;

  TLegend = class (TExcelBaseComponent)
   {TODO 2 -oTC -cCompletion : add properties: Border, Fill, Height, Position, Font, LegendEntries  }
  end;

  TLegendEntries = class( TDispCollection)
    {TODO 2 -oTC -cCompletion : TLegendEntries}
  end;

  TLegendEntry = class(TExcelBaseComponent)
    {TODO 2 -oTC -cCompletion : TLegendEntry}
  end;

  TChart = class( TAbstractSheet)
    {TODO 2 -oTC -cCompletion :
       PlotBy, HasAxis, Location, Axes, Shapes, Export, PlotArea }
  private
    function GetChartTitle: TChartTitle;
    function GetChartType: TChartType;
    function GetLegend: TLegend;
    procedure SetChartType(const AValue: TChartType);
  public
    property ChartTitle: TChartTitle  read GetChartTitle;
    property ChartType: TChartType  read GetChartType write SetChartType;
    property Legend: TLegend read GetLegend;
    function LegendEntry(anIndex:Integer): TLegendEntry;
    function LegendEntries: TLegendEntries;
    procedure SetSourceData( aRange: TRange; aPlotBy: TPlotBy);
  end;

  { TCharts }
  TCharts = class( TDispCollection)
  public
    Constructor Create( anOwner : TComponent); override; overload;
    Constructor Create( anOwner : TComponent; aDispatch : IDispatch); overload;
    function GetItem( index: OleVariant): TChart; override;
    function Add: TChart;
  end;


  { TWorkBook }
  TWorkBook = class( TDispComponent)
  public
    procedure Close( SaveChanges: Boolean);
    procedure Save;
    procedure SaveAs( aFileName : String);
    function Sheets: TSheets;
    function Charts: TCharts;
  end;

  { TWorkBooks }
  TWorkBooks = class( TDispComponent)
  public
    function Add : TWorkBook;
    function Open( aFileName: String) : TWorkBook;
  end;

  { TExcelApplication }
  TExcelApplication = class(TDispServer)
  private
    FWorkBooks: TWorkBooks;
    function GetDisplayAlerts: Boolean;
    function GetWorkBooks: TWorkBooks;
    procedure SetDisplayAlerts(const AValue: Boolean);
  public
    Constructor Create( anOwner : TComponent); override;
    Destructor Destroy; override;
    function Selection: TRange;
  published
    property WorkBooks: TWorkBooks read GetWorkBooks;
    property DisplayAlerts: Boolean read GetDisplayAlerts write SetDisplayAlerts;
   end;


implementation

{ TExcelApplication }
function TExcelApplication.GetWorkBooks: TWorkBooks;
var
  aDisp : IDispatch;
begin
  if not assigned( FWorkBooks) then
  begin
    //get the property,
    //return the IDispatch
    //create the Workbook and return it
    GetProperty( DISPID_WORKBOOKS, aDisp);
    FWorkBooks := TWorkBooks.Create( self, aDisp);
  end;
  result := FWorkBooks;
end;

function TExcelApplication.GetDisplayAlerts: Boolean;
begin
  Result:=GetProperty( DISPID_DISPLAYALERTS);
end;

procedure TExcelApplication.SetDisplayAlerts(const AValue: Boolean);
begin
  PutProperty( DISPID_DISPLAYALERTS,  aValue);
end;

constructor TExcelApplication.Create( anOwner:TComponent);
begin
  inherited Create(anOwner);
  ServerName := 'Excel.Application';
end;

destructor TExcelApplication.Destroy;
begin
  inherited Destroy;
end;

function TExcelApplication.Selection: TRange;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_SELECTION, aVar);
  { TODO 3 -oTC -cCompletion : the exact type of the selection object
    depends on the selected object. If Cells are selected the return is a range,
    and this is what is coded. Other types are unknown. }
  result := TRange.Create(self, aVar);
end;



{ TWorkBooks }

function TWorkBooks.Add: TWorkBook;
var
  aVar :IDispatch;
begin
  aVar := CallFunctionDisp( DISPID_ADD);
  result := TWorkBook.Create(self, aVar) ;
end;

function TWorkBooks.Open(aFileName: String): TWorkBook;
var
  aVar :IDispatch;
begin
  aVar := CallFunctionDisp( DISPID_ADD, [aFileName]);
  result := TWorkBook.Create(self, aVar) ;
end;

{ TWorkBook }

procedure TWorkBook.Close( SaveChanges: Boolean);
var
  varSaves : OleVariant;
begin
  varSaves := SaveChanges;
  CallProcedure( DISPID_SAVEAS, [varSaves]);
end;

procedure TWorkBook.Save;
begin
  CallProcedure(DISPID_SAVE);
end;

procedure TWorkBook.SaveAs(aFileName: String);
var
  varFileName : OleVariant;
begin
  varFileName := aFileName;
  CallProcedure( DISPID_SAVEAS, [varFileName]);
end;

function TWorkBook.Sheets: TSheets;
var
  aVar :IDispatch;
begin
  GetProperty(DISPID_SHEETS, aVar);
  result := TSheets.Create(self, aVar) ;
end;

function TWorkBook.Charts: TCharts;
var
  aVar :IDispatch;
begin
  GetProperty(DISPID_CHARTS, aVar);
  result := TCharts.Create(self, aVar) ;
end;

function TChart.GetChartTitle: TChartTitle;
var
  aVar :IDispatch;
begin
  GetProperty(DISPID_CHARTTITLE, aVar);
  result := TChartTitle.Create(self, aVar);
end;

function TChart.GetChartType: TChartType;
begin
  result := GetProperty( DISPID_CHARTTYPE);
end;

function TChart.GetLegend: TLegend;
var
  aVar :IDispatch;
begin
  GetProperty(DISPID_LEGEND, aVar);
  result := TLegend.Create(self, aVar);
end;

procedure TChart.SetChartType(const AValue: TChartType);
begin
  PutProperty(DISPID_CHARTTYPE, AValue);
end;

function TChart.LegendEntry(anIndex:Integer): TLegendEntry;
var
  aVar :IDispatch;
begin
  GetProperty(DISPID_LEGENDENTRIES, anIndex, aVar);
  result := TLegendEntry.Create(self, aVar);
end;

function TChart.LegendEntries: TLegendEntries;
var
  aVar :IDispatch;
begin
  GetProperty(DISPID_LEGENDENTRIES, aVar);
  result := TLegendEntries.Create(self, aVar);
end;

procedure TChart.SetSourceData(aRange: TRange; aPlotBy: TPlotBy);
begin
  CallProcedure( DISPID_SETSOURCEDATA, [aRange.ExcelObject, aPlotBy]);
end;

constructor TSheets.Create(anOwner: TComponent);
begin
  inherited Create(anOwner);
  ItemClassType := TSheet;
end;

constructor TSheets.Create(anOwner: TComponent; aDispatch: IDispatch);
begin
  inherited Create( anOwner, aDispatch);
end;

function TSheets.GetItem(index: OleVariant): TSheet;
begin
  Result := inherited GetItem(index) as TSheet;
end;

{ TSheets }
function TSheets.Sheet(index: OleVariant): TSheet;
var
  aVar :IDispatch;
begin
  GetProperty(DISPID_ITEM, index, aVar);
  result := TSheet.Create(self, aVar);
end;

{ TSheet }
function TSheet.Cells(RowIndex, ColumnIndex: Integer): TRange;
var
  aVar :IDispatch;
begin
  GetProperty(DISPID_CELLS, RowIndex, ColumnIndex, aVar);
  result := TRange.Create(self, aVar);
end;

function TSheet.Rows: TRange;
var
  aDisp : IDispatch;
begin
  GetProperty(DISPID_ROWS,  aDisp);
  result := TRange.Create( self, aDisp);
end;

//GetLastRowFilledIn
//Let LastRow = aSheet.Cells(Rows.Count, 3).End(xlUp).Row
function TSheet.GetLastRowFilledIn(ColumnIndex: Integer): Integer;
var
  aRows: TRange;
  aRowCount: Integer;
  aCell: TRange;
  aEndCell: TRange;
begin
  result := -1;
  aRows := Self.Rows;
  aRowCount := aRows.Count;
  aCell := Self.Cells( aRowCount, ColumnIndex);
  aEndCell := aCell.End_( xlUp);
  result :=  aEndCell.Row;
end;

procedure TSheet.Paste;
begin
  CallProcedure( DISPID_PASTE);
end;

function TSheet.Shapes: TShapes;
var
  aDisp : IDispatch;
begin
  GetProperty(DISPID_SHAPES,  aDisp);
  result := TShapes.Create( self, aDisp);
end;

function TSheet.Range(aRange: String): TRange;
var
  aDisp : IDispatch;
begin
  GetProperty(DISPID_RANGE, aRange, aDisp);
  result := TRange.Create( self, aDisp);
end;

function TSheet.Comments: TComments;
var
  aDisp : IDispatch;
begin
  GetProperty(DISPID_COMMENTS,  aDisp);
  result := TComments.Create( self, aDisp);
end;

{ TRange }

function TRange.GetBorders( index: TBorderIndex ): TBorder;
var
  aDisp : IDispatch;
  aBorders: TBorders;
  anIndex: Olevariant;
begin
  GetProperty( DISPID_BORDERS, aDisp);
  aBorders := TBorders.Create( self, aDisp);
  //TODO: find the bug that prevents getting a Border object back from the
  //Borders collection. Creates a HResult 800A03EC error!
  //-- this works: aBorders.ShowDispid('Item');
  //-- this doesn't: result := aBorders.GetItem(index);
  result := aBorders.GetItem(index);
  FreeAndNil( aBorders);
end;

function TRange.GetColumn: Long;
begin
  result := GetProperty( DISPID_COLUMN);
end;

function TRange.GetColumns(index: integer): TRange;
var
  aDisp : IDispatch;
begin
  GetProperty(DISPID_COLUMNS, index, aDisp);
  result := TRange.Create( self, aDisp);
end;

function TRange.GetColumnWidth: Single;
begin
  result := GetProperty( DISPID_COLUMNWIDTH);
end;

function TRange.GetComment: TComment;
var
  aDisp : IDispatch;
begin
  GetProperty(DISPID_COMMENT, aDisp);
  result := TComment.Create( self, aDisp);
end;

function TRange.GetFont: TFont;
var
  aDisp : IDispatch;
begin
  GetProperty(DISPID_FONT, aDisp);
  result := TFont.Create( self, aDisp);
end;

function TRange.GetFormula: String;
begin
  result := GetProperty( DISPID_FORMULA);
end;

function TRange.GetHeight: Long;
begin
  result := GetProperty( DISPID_HEIGHT);
end;

function TRange.GetHorizontalAlignment: THorizontalAlignment;
begin
  // DISPID_HORIZONTALALIGNMENT
  result := GetProperty( DISPID_HORIZONTALALIGNMENT);
end;

function TRange.GetInterior: TInterior;
var
  aDisp : IDispatch;
begin
  GetProperty(DISPID_INTERIOR, aDisp);
  result := TInterior.Create( self, aDisp);
end;

function TRange.GetNumberFormat: String;
begin
  result := GetProperty( DISPID_NUMBERFORMAT);
end;

function TRange.GetRow: Long;
begin
  result := GetProperty( DISPID_ROW);
end;

function TRange.GetRowHeight: Single;
begin
  result := GetProperty(DISPID_ROWHEIGHT);
end;

function TRange.GetRows(index: integer): TRange;
var
  aDisp : IDispatch;
begin
  GetProperty(DISPID_ROWS, index, aDisp);
  result := TRange.Create( self, aDisp);
end;

function TRange.GetText: String;
begin
  result := GetProperty(DISPID_TEXT);
end;

function TRange.GetValue: OleVariant;
begin
  result := GetProperty(DISPID_VALUE);
end;

function TRange.GetVerticalAlignment: TVerticalAlignment;
begin
  result := GetProperty( DISPID_VERTICALALIGNMENT);
end;

function TRange.GetWidth: Long;
begin
  result := GetProperty( DISPID_WIDTH);
end;

function TRange.GetWrapText: Boolean;
begin
  result := GetProperty( DISPID_WRAPTEXT);
end;

procedure TRange.SetColumnWidth(const AValue: Single);
begin
  PutProperty( DISPID_COLUMNWIDTH, AValue);
end;

procedure TRange.SetFormula(const AValue: String);
begin
  PutProperty(DISPID_FORMULA, [], AValue);
end;

procedure TRange.SetHorizontalAlignment(const AValue: THorizontalAlignment);
begin
  PutProperty( DISPID_HORIZONTALALIGNMENT, AValue);
end;

procedure TRange.SetNumberFormat(const AValue: String);
begin
  PutProperty( DISPID_NUMBERFORMAT, AValue)
end;

procedure TRange.SetRowHeight(const AValue: Single);
begin
  PutProperty( DISPID_ROWHEIGHT, AValue);
end;

procedure TRange.SetValue(const AValue: OleVariant);
begin
  //SetProperty('Value', AValue);
  PutProperty(DISPID_VALUE, [], AValue);
end;

procedure TRange.SetColumnWidth(const AValue: Long);
begin
  PutProperty( DISPID_COLUMNWIDTH, AValue);
end;

procedure TRange.SetVerticalAlignment(const AValue: TVerticalAlignment);
begin
  PutProperty( DISPID_VERTICALALIGNMENT, AValue);

end;

procedure TRange.SetWrapText(const AValue: Boolean);
begin
  PutProperty( DISPID_WRAPTEXT, AValue);
end;

function TRange.End_(aDirection: LongWord): TRange;
var
  aVar : IDispatch;
begin
  GetProperty( DISPID_END, aDirection, aVar);
  result := TRange.Create(Self.Owner, aVar);
end;

function TRange.GetCount: Long;
begin
  result := GetProperty( DISPID_COUNT);
end;

procedure TRange.AddComment(aText: String);
begin
  CallProcedure('AddComment', [aText] );
end;

procedure TRange.BorderAround(LineStyle: TLineDashStyle; Weight: TBorderWeight;
  ColorIndex: Short; Color: TRGBColor);
begin
  CallProcedure( DISPID_BORDERAROUND, [LineStyle, Weight, ColorIndex, Color]);
end;

procedure TRange.Select;
begin
  CallProcedure( DISPID_SELECT);
end;

procedure TRange.Copy;
begin
  CallProcedure( DISPID_COPY);
end;

procedure TRange.Replace(What, Replacement: String; Lookat: long;
  SearchOrder: long; MatchCase: Boolean);
begin
  CallProcedure(DISPID_REPLACE, [what, replacement, lookat, searchorder, matchcase]);
end;

{ TFont }

function TFont.GetBold: Boolean;
begin
  result := GetProperty(DISPID_BOLD);
end;

function TFont.GetColor: TRGBColor;
begin
  result := GetProperty(DISPID_COLOR);
end;

function TFont.GetFontName: String;
begin
  result := GetProperty(DISPID_NAME);
end;

function TFont.GetFontStyle: String;
begin
  result := GetProperty(DISPID_FONTSTYLE);
end;

function TFont.GetItalic: Boolean;
begin
  result := GetProperty(DISPID_ITALIC);
end;

function TFont.GetShadow: Boolean;
begin
  result := GetProperty(DISPID_SHADOW);
end;

function TFont.GetSize: Integer;
begin
  result := GetProperty(DISPID_SIZE);
end;

function TFont.GetSubscript: Boolean;
begin
  result := GetProperty(DISPID_SUBSCRIPT);
end;

function TFont.GetSuperscript: Boolean;
begin
result := GetProperty(DISPID_SUPERSCRIPT);
end;

function TFont.GetUnderlineStyle: TUnderlineStyle;
begin
  result := GetProperty(DISPID_UNDERLINESTYLE);
end;


procedure TFont.SetBold(const AValue: Boolean);
begin
  PutProperty( DISPID_BOLD, AValue);
end;

procedure TFont.SetColor(const AValue: TRGBColor);
begin
  PutProperty(DISPID_COLOR, AValue);
end;

procedure TFont.SetFontName(const AValue: String);
begin
  PutProperty( DISPID_NAME, AValue);
end;

procedure TFont.SetFontStyle(const AValue: String);
begin
  PutProperty( DISPID_FONTSTYLE, AValue);
end;

procedure TFont.SetItalic(const AValue: Boolean);
begin
  PutProperty(DISPID_ITALIC, AValue);
end;

procedure TFont.SetShadow(const AValue: Boolean);
begin
  PutProperty(DISPID_SHADOW, AValue);
end;

procedure TFont.SetSize(const AValue: Integer);
begin
  PutProperty( DISPID_SIZE, [], AValue);
end;

procedure TFont.SetSubscript(const AValue: Boolean);
begin
  PutProperty(DISPID_SUBSCRIPT, AValue);
end;

procedure TFont.SetSuperscript(const AValue: Boolean);
begin
  PutProperty(DISPID_SUPERSCRIPT, AValue);
end;

procedure TFont.SetUnderlineStyle(const AValue: TUnderlineStyle);
begin
  PutProperty(DISPID_UNDERLINESTYLE, AValue);
end;

{ TInterior }

function TInterior.GetColor: TRGBColor;
begin
  result := GetProperty(DISPID_COLOR);
end;

procedure TInterior.SetColor(const AValue: TRGBColor);
begin
  PutProperty(DISPID_COLOR, AValue);
end;

{ TShapes }

constructor TShapes.Create(anOwner: TComponent);
begin
  inherited Create(anOwner);
  ItemClassType := TShape;
end;

constructor TShapes.Create(anOwner: TComponent; aDispatch: IDispatch);
begin
  inherited Create(anOwner, aDispatch);
end;

function TShapes.GetItem(index: OleVariant): TShape;
begin
  result := inherited GetItem( index) as TShape;
end;

function TShapes.AddShape(aShapeType: TAutoShapeType; left, top, width,
  height: Single): TShape;
var
  newShapeDispatch: IDispatch;
begin
  newShapeDispatch := CallFunctionDisp( DISPID_ADDSHAPE, [aShapeType, left, top, width, height]);
  result := TShape.Create( Self, newShapeDispatch);
end;

function TShapes.AddLine(BeginX, BeginY, EndX, Endy: Single): TShape;
var
  newShapeDispatch: IDispatch;
begin
  newShapeDispatch := CallFunctionDisp( DISPID_ADDLINE, [BeginX, BeginY, EndX, Endy]);
  result := TShape.Create( Self, newShapeDispatch);
end;

function TShapes.AddConnector(AType: TConnectorType; BeginX, BeginY, Width, Height
  : Single): TShape;
var
  newShapeDispatch: IDispatch;
begin
  newShapeDispatch := CallFunctionDisp( DISPID_ADDCONNECTOR, [AType, BeginX, BeginY, Width, Height]);
  result := TShape.Create( Self, newShapeDispatch);
end;

{ TExcelComponent }

function TExcelComponent.GetVisible: Boolean;
begin
  result := GetProperty( DISPID_VISIBLE);
end;

procedure TExcelComponent.SetVisible(const AValue: Boolean);
begin
  PutProperty( DISPID_VISIBLE, BoolToVar(aValue));
end;

{ TShape }

function TShape.GetHeight: Integer;
begin
  result := GetProperty(DISPID_HEIGHT);
end;

function TShape.GetLeft: Integer;
begin
  result := GetProperty(DISPID_LEFT);
end;

function TShape.GetTop: Integer;
begin
  result := GetProperty(DISPID_TOP);
end;

function TShape.GetWidth: Integer;
begin
  result := GetProperty(DISPID_WIDTH);
end;

procedure TShape.SetHeight(const AValue: Integer);
begin
  PutProperty(DISPID_HEIGHT, AValue);
end;

procedure TShape.SetLeft(const AValue: Integer);
begin
  PutProperty(DISPID_LEFT, AValue);
end;

procedure TShape.SetTop(const AValue: Integer);
begin
  PutProperty(DISPID_TOP, AValue);
end;

procedure TShape.SetWidth(const AValue: Integer);
begin
  PutProperty(DISPID_WIDTH, AValue);
end;

function TShape.Fill: TFillFormat;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_FILL, aVar);
  result := TFillFormat.Create(self, aVar);
end;

function TShape.Line: TLineFormat;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_LINE, aVar);
  result := TLineFormat.Create(self, aVar);
end;

function TShape.TextFrame: TTextFrame;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_TEXTFRAME, aVar);
  result := TTextFrame.Create(self, aVar);
end;

function TShape.ConnectorFormat: TConnectorFormat;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_CONNECTORFORMAT, aVar);
  result := TConnectorFormat.Create(self, aVar);
end;

function TShape.Connector: TTriState;
begin
  Result:=GetProperty( DISPID_CONNECTOR);
end;

procedure TShape.RerouteConnections;
begin
  CallProcedure(DISPID_REROUTECONNECTIONS);
end;

{ TXlsCustomShape }

function TXlsCustomShape.GetVisible: TTriState;
begin
  Result:=GetProperty( DISPID_VISIBLE_FILL);
end;

procedure TXlsCustomShape.SetVisible(const AValue: TTriState);
begin
  PutProperty( DISPID_VISIBLE_FILL,  aValue);
end;

{ TXlsCustomFormat }

function TXlsCustomFormat.GetBackColor: TColorFormat;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_BACKCOLOR, aVar);
  result := TColorFormat.Create(self, aVar);
end;

function TXlsCustomFormat.GetTransparency: Double;
begin
  Result:=GetProperty( DISPID_TRANSPARENCY);
end;

procedure TXlsCustomFormat.SetTransparency(const AValue: Double);
begin
  PutProperty( DISPID_TRANSPARENCY,  aValue);
end;

function TXlsCustomFormat.GetForeColor: TColorFormat;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_FORECOLOR, aVar);
  result := TColorFormat.Create(self, aVar);
end;

{ TExcelBaseComponent }

procedure TExcelBaseComponent.Delete;
begin
  CallProcedure(DISPID_DELETE);
end;

{ TColorFormat }

function TColorFormat.GetRGB: TRGBColor;
begin
  result := GetProperty( DISPID_RGB);
end;

function TColorFormat.GetSchemeColor: Integer;
begin
  Result:=GetProperty( DISPID_SCHEMECOLOR);
end;

procedure TColorFormat.SetRGB(const AValue: TRGBColor);
begin
  PutProperty( DISPID_RGB,  aValue);
end;

procedure TColorFormat.SetSchemeColor(const AValue: Integer);
begin
  PutProperty( DISPID_SCHEMECOLOR,  aValue);
end;

{ TLineFormat }

function TLineFormat.GetDashStyle: TMSOLineDashStyle;
begin
  Result:=GetProperty( DISPID_LINEDASHSTYLE);
end;

function TLineFormat.GetStyle: TLineStyle;
begin
  Result:=GetProperty( DISPID_LINESTYLE);
end;

function TLineFormat.GetWeight: Single;
begin
  Result:=GetProperty( DISPID_WEIGHT);
end;

procedure TLineFormat.SetDashStyle(const AValue: TMSOLineDashStyle);
begin
  PutProperty( DISPID_LINEDASHSTYLE,  aValue);
end;

procedure TLineFormat.SetStyle(const AValue: TLineStyle);
begin
  PutProperty( DISPID_LINESTYLE,  aValue);
end;

procedure TLineFormat.SetWeight(const AValue: Single);
begin
  PutProperty( DISPID_WEIGHT,  aValue);
end;

function TLineFormat.GetForeColor: TColorFormat;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_FORECOLOR_LINE, aVar);
  result := TColorFormat.Create(self, aVar);
end;

{ TTextFrame }

function TTextFrame.GetAutosize: Boolean;
begin
  Result:=GetProperty( DISPID_AUTOSIZE);
end;

procedure TTextFrame.SetAutosize(const AValue: Boolean);
begin
  PutProperty( DISPID_AUTOSIZE,  aValue);
end;

function TTextFrame.Characters: TCharacters;
var
  aVar: IDispatch;
begin
  aVar := CallFunctionDisp(DISPID_CHARACTERS);
  result := TCharacters.Create(self, aVar);
end;

{ TCharacters }

function TCharacters.GetText: String;
begin
  Result:=GetProperty( DISPID_TEXT);
end;

procedure TCharacters.SetText(const AValue: String);
begin
  PutProperty( DISPID_Text,  aValue);
end;

function TCharacters.Font: TFont;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_FONT, aVar);
  result := TFont.Create(self, aVar);
end;

{ TComment }

procedure TComment.Delete;
begin
  CallProcedure(DISPID_DELETE);
end;

{ TComments }

constructor TComments.Create(anOwner: TComponent);
begin
  inherited Create(anOwner);
  ItemClassType := TComment;
end;

constructor TComments.Create(anOwner: TComponent; aDispatch: IDispatch);
begin
  inherited Create(anOwner, aDispatch);
end;

{ TFillFormat }

procedure TFillFormat.Solid;
begin
  CallProcedure(DISPID_SOLID);
end;

procedure TFillFormat.TwoColorGradient(aGradientStyle: TGradientStyle;
  aVariant: Integer);
begin
  CallProcedure( DISPID_TWOCOLORGRADIENT, [aGradientStyle, aVariant]);
end;

procedure TFillFormat.UserPicture(aPictureFileName: String);
begin
  CallProcedure( DISPID_USERPICTURE, [aPictureFileName]);
end;

{ TConnectorFormat }

procedure TConnectorFormat.BeginConnect(aShape: TShape; aConnectionSite: Long);
begin
  CallProcedure( DISPID_BEGINCONNECT, [aShape.ExcelObject, aConnectionSite]);
end;

procedure TConnectorFormat.EndConnect(aShape: TShape; aConnectionSite: Long);
begin
  CallProcedure( DISPID_ENDCONNECT, [aShape.ExcelObject, aConnectionSite]);
end;

procedure TConnectorFormat.BeginDisconnect;
begin
  CallProcedure( DISPID_BEGINDISCONNECT);
end;

procedure TConnectorFormat.EndDisconnect;
begin
  CallProcedure( DISPID_ENDDISCONNECT);
end;

{ TCharts }

constructor TCharts.Create(anOwner: TComponent);
begin
  inherited Create(anOwner);
  ItemClassType := TChart;
end;

constructor TCharts.Create(anOwner: TComponent; aDispatch: IDispatch);
begin
  inherited Create( anOwner, aDispatch);
end;

function TCharts.GetItem(index: OleVariant): TChart;
begin
  Result := inherited GetItem(index) as TChart
end;

function TCharts.Add: TChart;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_ADD, aVar);
  result := TChart.Create(self, aVar);
end;

{ TAbstractSheet }

function TAbstractSheet.GetType: TSheetType;
begin
  result := GetProperty(DISPID_TYPE);
end;

procedure TAbstractSheet.SetType(const AValue: TSheetType);
begin
  PutProperty( DISPID_TYPE, AValue);
end;

{ TChartTitle }

function TChartTitle.GetBorder: TBorder;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_Border, aVar);
  result := TBorder.Create(self, aVar);
end;

function TChartTitle.GetCharacters: TCharacters;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_CHARACTERS, aVar);
  result := TCharacters.Create(self, aVar);
end;

function TChartTitle.GetFill: TFillFormat;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_FILL, aVar);
  result := TFillFormat.Create(self, aVar);
end;

function TChartTitle.GetInterior: TInterior;
var
  aVar: IDispatch;
begin
  GetProperty(DISPID_INTERIOR, aVar);
  result := TInterior.Create(self, aVar);
end;

function TChartTitle.GetText: String;
begin
  Result:=GetProperty( DISPID_TEXT);
end;

procedure TChartTitle.SetText(const AValue: String);
begin
  PutProperty( DISPID_Text,  aValue);
end;

{ TBorders }

constructor TBorders.Create(anOwner: TComponent);
begin
  inherited Create(anOwner);
  ItemClassType := TBorder;
end;

constructor TBorders.Create(anOwner: TComponent; aDispatch: IDispatch);
begin
  inherited Create(anOwner, aDispatch);
end;

function TBorders.GetItem(index: OleVariant): TBorder;
begin
  Result:=inherited GetItem(index) as TBorder;
end;

{ TBorder }

function TBorder.GetLineStyle: TLineDashStyle;
begin
  Result:=GetProperty( DISPID_LINESTYLE);
end;

procedure TBorder.SetLineStyle(const AValue: TLineDashStyle);
begin
  PutProperty( DISPID_LINESTYLE,  aValue);
end;

end.

