{******************************************************************************}
{                                                                              }
{ Excel Interface Component                                                    }
{ Component ExcelUtilities.pas                                                 }
{ This interface component can be downloaded at www.tcoq.org, component page.  }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License Version  }
{ 1.1 (the "License"); you may not use this file except in compliance with the }
{ License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ The Original Code is TestCaseExcel.pas                                      }
{                                                                              }
{ The Initial Developer of the Original Code is Thierry.Coq                    }
{ thierry.coq(at)centraliens.net                                               }
{ Portions created by this individual are Copyright (C)                        }
{ 2000-2008 of this individual.                                                }
{                                                                              }
{ Last modified: 2009/07/25                                                    }
{ Version 0.4                                                                  }
{                                                                              }
{******************************************************************************}
unit TestCaseExcel;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, fpcunit, testutils, testregistry, ExcelUtilities;

const
  TestFileName = 'c:\Donnees\test.xls';

type

  { TTestCaseExcel }

  TTestCaseExcel= class(TTestCase)
  private
    aSheet : TSheet;
    aCell : TRange;
  protected
    procedure SetUp; override; 
    procedure TearDown; override; 
  public
    Constructor Create; override;
    Destructor Destroy; override;
  published
    procedure TestHookUp;
    procedure TestCheckSheetType;
    procedure TestFirstExcelSimpleTest;
    procedure TestFontFormatting;
    procedure TestFontFormattingAdvanced;
    procedure TestColumnFormatting;
    procedure TestRowFormatting;
    procedure TestAddComment;
    procedure TestBorderAround;
    procedure TestBorders;
    procedure TestWrapText;
    procedure TestVerticalAlignment;
    procedure TestHorizontalAlignment;
    procedure TestNumberFormat;
    procedure TestAddShapeDispid;
    procedure TestShapes;
    procedure TestShapeFilling;
    procedure TestShapeAdvancedFilling;
    procedure TestShapePictureFilling;
    procedure TestShapesOtherShapes;
    procedure TestShapesFlowShapes;
    procedure TestShapeLine;
    procedure TestShapeLineAdvanced;
    procedure TestShapeCharacters;
    procedure TestLineNew;
    procedure TestConnectorNew;
    procedure TestSelection;
    procedure TestChartAdd;
    procedure TestSave;
  end;

implementation

uses
  Windows;

// Unit variables: common to all tests, in order to have only one workbook
// for most of the testing.
Var
  ExcelApp : TExcelApplication;
  ExcelWbs : TWorkBooks;
  ExcelWb  : TWorkBook;
  ExcelSheets : TSheets;
  ExcelSheet1 : TSheet;

procedure TTestCaseExcel.TestHookUp; 
begin
  AssertTrue('Excel should be visible now!', Assigned( ExcelApp));
end;

procedure TTestCaseExcel.TestCheckSheetType;
begin
  AssertTrue('Sheet is a worksheet', aSheet.Type_ = xlWorksheet);
end;

procedure TTestCaseExcel.TestFirstExcelSimpleTest;
var
  aValue : OleVariant;
begin
  aCell := aSheet.Cells(1, 1);
  aCell.Value := 10;
  aValue := aCell.Value;
  AssertTrue( 'checking values can be get and set', aValue = 10);

  aCell := aSheet.Cells(2,1);
  aCell.Formula := '=A1+10';
  aValue := aCell.Value;
  AssertTrue( 'checking formulas can be get and set', aValue = 20);

  aCell.Interior.Color := $FF0000; //Blue
  AssertTrue( 'color has been set', aCell.Interior.Color = $FF0000);

  aCell := aSheet.Cells(3,1);
  aCell.Value := '=A2+10';
end;

procedure TTestCaseExcel.TestFontFormatting;
begin
  // FirstExcelSimpleTest shall have been successful, before this one can be.
  // ========================================================================

  // Setting Font to bold.
  aCell := aSheet.Cells(4,1);
  aCell.Value := 'Bold';
  aCell.Font.Bold:= true;
  AssertTrue('Font is bold', aCell.Font.Bold);

  // Setting Font Size.
  aCell := aSheet.Cells(5,1);
  aCell.Value := 'Big Size';
  aCell.Font.Size := 20;
  AssertTrue('Font size is 20', aCell.Font.Size = 20);

  // Setting Font Color
  aCell := aSheet.Cells(6,1);
  aCell.Value := 'Color Blue';
  aCell.Font.Color := $FF0000;
  AssertTrue('Font is blue', aCell.Font.Color = $FF0000);

end;

procedure TTestCaseExcel.TestFontFormattingAdvanced;
begin
  // Setting Font in Italic
  aCell := aSheet.Cells(4,2);
  aCell.Value := 'Italics';
  aCell.Font.Italic := true;
  AssertTrue('Font is in Italics', aCell.Font.Italic);
  freeAndNil(aCell);

  // Setting Another Font
  aCell := aSheet.Cells(5,2);
  aCell.Value := 'Times New Roman';
  aCell.Font.FontName := 'Times New Roman';
  AssertTrue('Font is Times New Roman', aCell.Font.FontName = 'Times New Roman');
  freeAndNil(aCell);

  // Setting Subscript
  aCell := aSheet.Cells(6,2);
  aCell.Value := 'Subscript';
  aCell.Font.Subscript := true;
  AssertTrue('Font is subscript', aCell.Font.Subscript);
  freeAndNil(aCell);

  // Setting Superscript
  aCell := aSheet.Cells(4,3);
  aCell.Value := 'Superscript';
  aCell.Font.Superscript := true;
  AssertTrue('Font is superscript', aCell.Font.Superscript);
  freeAndNil(aCell);

  // Setting Shadow
  aCell := aSheet.Cells(5,3);
  aCell.Value := 'Shadow';
  aCell.Font.Shadow := true;
  AssertTrue('Font is superscript', aCell.Font.Shadow);
  freeAndNil(aCell);

  // Setting FontStyle
  aCell := aSheet.Cells(6,3);
  aCell.Value := 'Bold Italic';
  aCell.Font.FontStyle := 'bold italic';
  AssertTrue('Font is in Italics', aCell.Font.Italic);
  AssertTrue('Font is bold', aCell.Font.Bold);
  freeAndNil(aCell);

  // Setting Underline
  aCell := aSheet.Cells(4,4);
  aCell.Value := 'Underline';
  aCell.Font.Underline := xlUnderlineStyleSingle;
  AssertTrue('Font is underline', aCell.Font.Underline = xlUnderlineStyleSingle);
  freeAndNil(aCell);

  aCell := aSheet.Cells(5,4);
  aCell.Value := 'Double Underline';
  aCell.Font.Underline := xlUnderlineStyleDouble;
  AssertTrue('Font is double underline', aCell.Font.Underline = xlUnderlineStyleDouble);
  freeAndNil(aCell);

  aCell := aSheet.Cells(6,4);
  aCell.Value := 'Single Underline Accounting';
  aCell.Font.Underline := xlUnderlineStyleSingleAccounting;
  AssertTrue('Font is single underline accounting', aCell.Font.Underline = xlUnderlineStyleSingleAccounting);
  freeAndNil(aCell);

end;

procedure TTestCaseExcel.TestColumnFormatting;
begin
  aCell := aSheet.Cells(1,2);
  aCell.ColumnWidth := 100;
  AssertTrue('Column width', aCell.ColumnWidth = 100);
  freeAndNil(aCell);
end;

procedure TTestCaseExcel.TestRowFormatting;
begin
  aCell := aSheet.Cells(1,1);
  aCell.RowHeight := 60;
  AssertTrue('Row height', aCell.RowHeight = 60);
  freeAndNil(aCell);
end;

procedure TTestCaseExcel.TestAddComment;
var
  aRange:TRange;
  aComment: TComment;
  aComments: TComments;
  commentCount: integer;
begin
  aCell := aSheet.Cells(1,1);
  aCell.AddComment('this is a comment');
  aRange := aSheet.Range('A1');
  aComment := aRange.Comment;
  AssertNotNull('Comment object is assigned', aComment);
  aComment.Visible := true;
  AssertTrue('Comment object is visible', aComment.Visible);
  freeAndNil(aRange);

  aComments := aSheet.Comments;
  commentCount := aComments.Count;
  aRange := aSheet.Range('A2');
  aRange.AddComment('example comment');
  AssertTrue('Number of comments is one more', commentCount+1 = aComments.Count);
  aComment := aRange.Comment;
  aComment.Delete;
  AssertTrue('Number of comments is bacl to previous', commentCount = aComments.Count);

  freeAndNil(aCell);
  freeAndNil(aRange);
  freeAndNil(aComments);

end;

procedure TTestCaseExcel.TestBorderAround;
begin
  aCell := aSheet.Cells(1,1);
  aCell.BorderAround( lsDash, bwThick, 1, $FF0000);
  freeAndNil(aCell);
end;

procedure TTestCaseExcel.TestBorders;
var
  aBorder : TBorder;
  aSelection: TRange;
begin
  aCell := aSheet.Cells(2, 1);
  aCell.BorderAround( lsDash, bwThick, 1, $FF0000);
  aCell.Select;

  (*TODO: Correct bug in getting the border object. Not working now.
  aSelection := ExcelApp.Selection;
  aBorder  := aSelection.Borders[xlEdgeLeft];
  aBorder.ShowDispid('LineStyle');
  aBorder.LineStyle:= lsDouble;
  AssertNotNull( aBorder);
  *)
end;

procedure TTestCaseExcel.TestWrapText;
begin
  aCell := aSheet.Cells(7,1);
  aCell.Value := 'This is a very long text, that should be wrapped';
  aCell.WrapText:=true;

  aCell := aSheet.Cells(8,1);
  aCell.Value := 'This is a very long text, that should NOT be wrapped';
  aCell.WrapText:=false;
end;

procedure TTestCaseExcel.TestVerticalAlignment;
begin
  aCell := aSheet.Cells(9,1);
  aCell.Value := 'VA : Top';
  aCell.RowHeight := 60;
  aCell.VerticalAlignment:=VAlignTop;

  aCell := aSheet.Cells(10,1);
  aCell.Value := 'VA : Center';
  aCell.RowHeight := 60;
  aCell.VerticalAlignment:=VAlignCenter;

end;

procedure TTestCaseExcel.TestHorizontalAlignment;
begin
  aCell := aSheet.Cells(9,2);
  aCell.Value := 'HA: Center';
  aCell.RowHeight := 60;
  aCell.HorizontalAlignment:=HAlignCenter;

  aCell := aSheet.Cells(10,2);
  aCell.Value := 'HA: Left';
  aCell.RowHeight := 60;
  aCell.HorizontalAlignment:=HAlignLeft;

  aCell := aSheet.Cells(11,2);
  aCell.Value := 'HA: Right';
  aCell.RowHeight := 60;
  aCell.HorizontalAlignment:=HAlignRight;
end;

procedure TTestCaseExcel.TestNumberFormat;
begin
  aCell := aSheet.Cells(12,1);
  aCell.Value := 1.251;
  aCell.NumberFormat := 'hh:mm:ss';
end;

procedure TTestCaseExcel.TestAddShapeDispid;
var
  shapeId: integer;
  shapes: TShapes;
begin
  //Get the Dispid, confirm it is correct
  shapeId := aSheet.GetIdsOfNames('Shapes');
  AssertEquals('Shapes declared Dispid :'+intToStr(shapeId), DISPID_SHAPES, shapeid);

  shapes := aSheet.Shapes;
  AssertTrue('Shapes collection has been initialized', Assigned(shapes));

end;

procedure TTestCaseExcel.TestSave;
begin
  ExcelApp.DisplayAlerts:=false;
  ExcelWB.Save;
  ExcelApp.DisplayAlerts:=true;
  AssertTrue('Save has been completed', True);
end;

procedure TTestCaseExcel.TestShapes;
var
  shapeId: integer;
  shapes: TShapes;
  shapeCountBefore: integer;
  shapeCountAfter: integer;
  newShape: TShape;
  aValue: Integer;
const
  CstLeft = 10;
  CstTop  = 20;
  CstWidth = 30;
  CstHeight = 40;
begin
  //Get the Dispid, confirm it is correct
  shapeId := aSheet.GetIdsOfNames('Shapes');
  AssertEquals('Shapes declared Dispid :'+intToStr(shapeId), DISPID_SHAPES, shapeid);

  shapes := aSheet.Shapes;
  AssertTrue('Shapes collection has been initialized', Assigned(shapes));

  shapeCountBefore := shapes.Count;
  AssertTrue('Shapes collection should have more than 0 shapes, has '+intToStr(shapeCountBefore), shapeCountBefore>=0);

  // add a new shape,
  newShape := shapes.AddShape( msoShapeRectangle, CstLeft, CstTop, CstWidth, CstHeight);

  // The number of shapes should be now 2
  shapeCountAfter := shapes.Count;
  AssertTrue('Shapes collection should have 1 new shape, has '+intToStr(shapeCountAfter), shapeCountAfter=shapeCountBefore+1);

  // Check the left, top, width, height
  aValue := newShape.Left;
  AssertTrue('Shape Left should be '+IntToStr(CstLeft)+', but is '+IntToStr(aValue), aValue=CstLeft);

  aValue := newShape.Top;
  AssertTrue('Shape Top should be '+IntToStr(CstTop)+', but is '+IntToStr(aValue), aValue=CstTop);

  aValue := newShape.Width;
  AssertTrue('Shape Width should be '+IntToStr(CstWidth)+', but is '+IntToStr(aValue), aValue=CstWidth);

  aValue := newShape.Height;
  AssertTrue('Shape Height should be '+IntToStr(CstHeight)+', but is '+IntToStr(aValue), aValue=CstHeight);

end;

procedure TTestCaseExcel.TestShapeFilling;
var
  newShape: TShape;
  shapes: TShapes;
  aFill : TFillFormat;
  aForeColor : TColorFormat;
begin
  //building a new shape for filling
  shapes := aSheet.Shapes;
  newShape := shapes.AddShape( msoShapeRectangle, 50, 10, 10, 10);

  //Checking that the filling can be changed.
  aFill := newShape.Fill;
  AssertNotNull ('Fill should be assigned!', aFill);
  aFill.Visible := msoFalse;
  AssertTrue('Fill.Visible should be false', aFill.Visible=msoFalse);
  aFill.Visible := msoTrue;
  aFill.Transparency := 0.5; //50%
  AssertTrue('Fill.Visible should be false', aFill.Transparency=0.5);

  //Changing the color of the shape.
  aFill.Solid;
  aForeColor := aFill.ForeColor;
  aForeColor.RGB := $FF0000;

  //another way to change the color of the shape:
  aForeColor.SchemeColor := 50; //we can also change the color if it's standard.

end;

procedure TTestCaseExcel.TestShapeAdvancedFilling;
var
  newShape: TShape;
  shapes: TShapes;
  aFill : TFillFormat;
  aForeColor : TColorFormat;
  aBackColor : TColorFormat;
begin
  //building a new shape for filling
  shapes := aSheet.Shapes;
  newShape := shapes.AddShape( msoShapeRectangle, 130, 10, 10, 10);

  //Checking that the filling can be changed.
  aFill := newShape.Fill;
  aFill.Visible := msoTrue;

  //Changing the color of the shape.
  aForeColor := aFill.ForeColor;
  aForeColor.RGB := $FF0000;
  aBackColor := aFill.BackColor;
  aBackColor.RGB := $00FF00;
  aFill.TwoColorGradient( msoGradientHorizontal, 1);

end;

procedure TTestCaseExcel.TestShapePictureFilling;
var
  newShape: TShape;
  shapes: TShapes;
  aFill : TFillFormat;
begin
  //building a new shape for filling
  shapes := aSheet.Shapes;
  newShape := shapes.AddShape( msoShapeRectangle, 150, 10, 50, 50);
  aFill := newShape.Fill;
  aFill.Visible := msoTrue;
  aFill.UserPicture('C:\Program Files\Microsoft Office\OFFICE11\BITMAPS\STYLES\GLOBE.WMF');
end;

procedure TTestCaseExcel.TestShapesOtherShapes;
var
  newShape: TShape;
  shapes: TShapes;
begin
  //building a new shape
  shapes := aSheet.Shapes;
  newShape := shapes.AddShape( msoShapeDiamond, 210, 10, 10, 10);
  newShape := shapes.AddShape( msoShapeOval, 230, 10, 10, 10);
  newShape := shapes.AddShape( msoShapeHexagon, 250, 10, 10, 10);
  newShape := shapes.AddShape( msoShapeArc, 270, 10, 10, 10);
  newShape := shapes.AddShape( msoShapeFlowchartProcess, 290, 10, 10, 10);
end;

procedure TTestCaseExcel.TestShapesFlowShapes;
var
  newShape: TShape;
  shapes: TShapes;
begin
  //building a new shape
  shapes := aSheet.Shapes;
  newShape := shapes.AddShape( msoShapeFlowchartAlternateProcess, 200, 200, 10, 10);
  newShape := shapes.AddShape( msoShapeFlowchartDecision, 220, 200, 10, 10);
  newShape := shapes.AddShape( msoShapeFlowchartDocument, 240, 200, 10, 10);
  newShape := shapes.AddShape( msoShapeFlowchartConnector, 260, 200, 10, 10);
  newShape := shapes.AddShape( msoShapeFlowchartOr, 280, 200, 10, 10);
  newShape := shapes.AddShape( msoShapeFlowchartMerge, 300, 200, 10, 10);
  newShape := shapes.AddShape( msoShapeFlowchartMagneticDisk, 320, 200, 10, 10);
  newShape := shapes.AddShape( msoShapeFlowchartDisplay, 340, 200, 10, 10);

end;

procedure TTestCaseExcel.TestShapeLine;
var
  newShape: TShape;
  shapes: TShapes;
  aLine : TLineFormat;
  aForeColor : TColorFormat;
begin
  //building a new shape for filling
  shapes := aSheet.Shapes;
  newShape := shapes.AddShape( msoShapeRectangle, 70, 10, 10, 10);

  //Changing the style and color of the line:
  aLine := newShape.Line;
  aLine.Weight := 2;
  aLine.Style := msoLineThickThin;
  aLine.DashStyle:=msoLineDash;
  aForeColor := aLine.ForeColor;
  aForeColor.RGB := $FF0000;

end;

procedure TTestCaseExcel.TestShapeLineAdvanced;
var
  newShape: TShape;
  shapes: TShapes;
  aLine : TLineFormat;
begin
  //building a new shape for filling
  shapes := aSheet.Shapes;
  newShape := shapes.AddShape( msoShapeRectangle, 210, 10, 20, 20);

  //Changing the dash style of the line:
  aLine := newShape.Line;
  aLine.DashStyle:=msoLineDash;
  aLine.Weight := 0.75;
  aLine.Style := msoLineSingle;
end;

procedure TTestCaseExcel.TestShapeCharacters;
var
  newShape: TShape;
  shapes: TShapes;
  aTextFrame: TTextFrame;
  aCharacters: TCharacters;
  aFont: TFont;
begin
  //building a new shape for filling
  shapes := aSheet.Shapes;
  newShape := shapes.AddShape( msoShapeRectangle, 90, 10, 50, 10);

  //newShape.ShowDispid('TextFrame');
  aTextFrame := newShape.TextFrame;
  aCharacters := aTextFrame.Characters;
  aCharacters.Text := 'Test';
  AssertTrue('Text should be test and is not!', aCharacters.Text='Test');
  aFont := aCharacters.Font;
  AssertNotNull( 'Font is assigned', aFont);
  aFont.Bold := true ;
  AssertTrue('Font is Bold', aFont.Bold);
  aFont.Shadow := true ;
  AssertTrue('Font is Shadow', aFont.Shadow);
  AssertFalse('Autosize not set', aTextFrame.Autosize);
  atextFrame.Autosize := True;
  AssertTrue('Autosize set', aTextFrame.Autosize);
end;

procedure TTestCaseExcel.TestLineNew;
var
  newLine: TShape;
  shapes: TShapes;
begin
  shapes := aSheet.Shapes;
  newLine := shapes.AddLine( 10, 10, 500, 100);
end;

procedure TTestCaseExcel.TestConnectorNew;
var
  newConnector, shape1, shape2: TShape;
  shapes: TShapes;
  aConnectorFormat: TConnectorFormat;
begin
  shapes := aSheet.Shapes;

  //Creating the shapes to connect to.
  shape1 := shapes.AddShape( msoShapeRectangle, 100, 300, 10, 10);
  shape2 := shapes.AddShape( msoShapeRectangle, 120, 300, 10, 10);

  //Creating the connector, and linking to the shapes.
  newConnector := shapes.AddConnector( msoConnectorStraight, 110, 305, 10, 0);
  AssertTrue('NewConnector is a connector', newConnector.Connector=msoTrue);
  aConnectorFormat := newConnector.ConnectorFormat;
  aConnectorFormat.BeginConnect(shape1, 1);
  aConnectorFormat.EndConnect(shape2, 1);
  newConnector.RerouteConnections;

  //This is how the connector is disconnected, if needed.
  aConnectorFormat.BeginDisconnect;
  aConnectorFormat.EndDisconnect;

  //reconnect again, to allow for human testing.
  aConnectorFormat.BeginConnect(shape1, 1);
  aConnectorFormat.EndConnect(shape2, 1);
  newConnector.RerouteConnections;
end;

procedure TTestCaseExcel.TestSelection;
var
  aSelect: TRange;
begin
  aSheet.Cells(1,1).Select;
  aSelect := ExcelApp.Selection;
  aSelect.Font.Bold := true;
  AssertTrue('cell(1,1) has a bold font', aSheet.Cells(1,1).Font.Bold);
  FreeAndNil(aSelect);
end;

procedure TTestCaseExcel.TestChartAdd;
var
  aChart : TChart;
  aRange: TRange;
begin
  //setting up the chart data. X'es:
  aSheet.Cells( 20, 3).Value := 'X';
  aSheet.Cells( 21, 3).Value := 1;
  aSheet.Cells( 22, 3).Value := 2.5;
  aSheet.Cells( 23, 3).Value := 4;
  aSheet.Cells( 26, 3).Value := 6;

  // and Y'es:
  aSheet.Cells( 20, 4).Value := 'Y';
  aSheet.Cells( 21, 4).Value := 1;
  aSheet.Cells( 22, 4).Value := 6;
  aSheet.Cells( 23, 4).Value := 14;
  aSheet.Cells( 26, 4).Value := 11;

  // setting up the range object to the data:
  aRange := aSheet.Range('C20:D26');

  aChart := ExcelWb.Charts.Add;
  aChart.ChartType := xlXYScatter;
  aChart.SetSourceData( aRange, xlColumns);
end;


procedure TTestCaseExcel.SetUp; 
begin
  aSheet := ExcelSheet1;
  AssertTrue('Excel sheet can be retrieved', Assigned(ExcelWB));
end;

procedure TTestCaseExcel.TearDown; 
begin
end;

constructor TTestCaseExcel.Create;
begin
  inherited Create;
end;

destructor TTestCaseExcel.Destroy;
begin
  inherited Destroy;
end;

initialization
  RegisterTest(TTestCaseExcel);

  // Initializing the common excel workbook:
  ExcelApp := TExcelApplication.Create(nil);
  ExcelApp.Active := True;
  ExcelApp.Visible := True;
  ExcelWbs := ExcelApp.WorkBooks;

  if FileExists( TestFileName) then
    ExcelWb := ExcelWbs.Open( TestFileName)
  else
  begin
    ExcelWb := ExcelWbs.Add;
    ExcelWB.SaveAs(TestFileName);
  end;

  ExcelSheets := ExcelWb.Sheets;
  ExcelSheet1 := ExcelSheets.Sheet('Sheet1');
finalization
  ExcelApp.Quit;
  FreeAndNil(ExcelApp);
end.

